/*===========================================================================
 *
 * File:	Dl_blowf.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	April 29, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __DL_BLOWF_H
#define __DL_BLOWF_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "dl_err.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/
	
	/* Array sizes */
  #define BFALG_PDATA_SIZE  18
  #define BFALG_SDATA_SIZE1 4
  #define BFALG_SDATA_SIZE2 256

  #define BFALG_KEY_MAXSIZE 56          /* 448 bits */

  #define BFALG_PDATA_LOOPSIZE 16

  #define BFALG_MIN_DATASIZE 8

/*==========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CBlowFishAlg Definition
 *
 * Description
 *
 *=========================================================================*/
class CBlowFishAlg {

  /*---------- Begin Private Class Members ----------------------*/
private:

		/* Class number arrays */
  dword m_PData[BFALG_PDATA_SIZE];
  dword m_SData[BFALG_SDATA_SIZE1][BFALG_SDATA_SIZE2];

  byte  m_PadByte;		/* Padding byte */


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Internal conversion method */
  dword Convert (const dword Val);

	/* Main encryption/decryption methods */
  void EncryptBlock (dword& XLeft, dword& XRight);
  void DecryptBlock (dword& XLeft, dword& XRight);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CBlowFishAlg (const TCHAR* pKeyName);
  virtual ~CBlowFishAlg() { Destroy(); }
  virtual void Destroy (void);

	/* Encrypt or descrypt two long numbers */
  void EncryptLong (dword& Value1, dword& Value2) { EncryptBlock(Value1, Value2); }
  void DecryptLong (dword& Value1, dword& Value2) { DecryptBlock(Value1, Value2); }

	/* Encrypt/decrypt an arbitrarily long string buffer in place */
  void EncryptBuffer (byte* pBuffer, int& BufferSize);
  void DecryptBuffer (byte* pBuffer, int& BufferSize);

  	/* Get class members */
  byte GetPadByte (void) { return (m_PadByte); }

	/* Initializes the algorithm with a new key string */
  void Initialize (const TCHAR* pKeyName);

	/* Performs a quick test of the algorithm */
  bool Test (void);
  bool StressTest (const long NumLoops = 100000);

	/* Set class members */
  void SetPadByte (const byte Char) { m_PadByte = Char; }

 };
/*===========================================================================
 *		End of Class CBlowFishAlg Definition
 *=========================================================================*/



#endif
/*===========================================================================
 *		End of File Dl_blowf.H
 *=========================================================================*/
