/*===========================================================================
 *
 * File:	Dfwoods.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	May 30, 2003

 *
 * Basic class for handling Daggerfall's WOODS.WLD file.
 *
 *=========================================================================*/
#ifndef __DFWOODS_H
#define __DFWOODS_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "genfile.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Array sizes */
  #define DFWOOD_NUMHEADERNULLS 28

	/* Pixel flags */
  #define DFWOOD_FLAG_DESERT		0x01	/* 002 */
  #define DFWOOD_FLAG_MOUNTAIN		0x02	/* 102 */
  #define DFWOOD_FLAG_TEMPERATE		0x04	/* 302 */
  #define DFWOOD_FLAG_JUNGLE		0x08	/* 402 */
  #define DFWOOD_FLAG_HEIGHT1		0x20
  #define DFWOOD_FLAG_HEIGHT2		0x40
  #define DFWOOD_FLAG_HEIGHT3		0x60
  #define DFWOOD_FLAG_HEIGHT4		0x80
  #define DFWOOD_FLAG_HEIGHT5		0xA0
  #define DFWOOD_FLAG_HEIGHT6		0xC0

	/* Climate/texture types */
  #define DFWOOD_TILESET_DESERT		2
  #define DFWOOD_TILESET_MOUNTAIN	102
  #define DFWOOD_TILESET_TEMPERATE	302
  #define DFWOOD_TILESET_JUNGLE		402

	/* General land types? */
  #define DFWOOD_LAND_TEMPERATE		0
  #define DFWOOD_LAND_MOUNTAIN		1
  #define DFWOOD_LAND_DESERT		2

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

	/* Header structure */
  typedef struct {
	long OffsetSize;
	long ImageWidth;
	long ImageHeight;
	long NullValue1;
	long Data1Offset;
	long Unknown1;
	long Unknown2;
	long SmallMapOffset;
	long NullValues[DFWOOD_NUMHEADERNULLS];
   } dfwoodsheader_t;

	/* Optional structure used to get individual map pixel data */
  typedef struct {
	byte	Color;	/* Gray scale map color */
	byte    Flags;
   } dfwoodspixshrt_t;

	/* Full pixel data structure */
  typedef struct {
	short	NumberSeed;
	long	NullValue1;
	short	ClimateType;
	char	LandType;
	char	LandHeight;
	long    NullValues[3];

	union {
		 byte MapData[5][5];
		 byte MapDataAll[25];
	 };
   } dfwoodspixel_t;
 	

#pragma pack(pop)
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CDfWoodsFile Definition
 *
 * Description
 *
 *=========================================================================*/
class CDfWoodsFile : public CGenFile {

  /*---------- Begin Private Class Members ----------------------*/
private:
  dfwoodsheader_t	m_Header;
  bool			m_HasHeader;

  byte*			m_pSmallMap;
  size_t		m_SmallMapSize;
  

  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Helper allocation methods */
  bool AllocateSmallMap (void);

  	/* Compute a pixel data offset */
  long GetPixelOffset (const int XPixel, const int YPixel);

	/* Helper input methods */
  bool ReadHeader   (void);
  bool ReadSmallMap (void);
  
  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CDfWoodsFile();
  virtual ~CDfWoodsFile() { Destroy(); }
  virtual void Destroy (void);

	/* Export map data to RAW data files */
  bool ExportSmallMap (const TCHAR* pFilename);
  bool ExportLargeMap (const TCHAR* pFilename);
  bool ExportClimate  (const TCHAR* pFilename);
  bool ExportType     (const TCHAR* pFilename);
  bool ExportHeight   (const TCHAR* pFilename);
  bool ExportSeed     (const TCHAR* pFilename);

	/* Get the small map data, load if required */
  byte* GetSmallMap (void);

	/* Allocate and read a short pixel data array */
  dfwoodspixshrt_t* ReadShortPixelData (void);

	/* Get class members */
  long  GetSmallMapWidth  (void) const { return (m_Header.ImageWidth); }
  long  GetSmallMapHeight (void) const { return (m_Header.ImageHeight); }
  long  GetSmallMapSize   (void) const { return (m_SmallMapSize); }

	/* Checks a pixel coordinate */
  bool IsValidPixel (const int X, const int Y);

	/* Read a map pixel value */
  bool ReadPixel    (dfwoodspixel_t& PixelData, const int XPixel, const int YPixel);

 };
/*===========================================================================
 *		End of Class CDfWoodsFile Definition
 *=========================================================================*/



#endif
/*===========================================================================
 *		End of File Dfwoods.H
 *=========================================================================*/
