/*===========================================================================
 *
 * File:	Dfinutil.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	January 18, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "dfinstall.h"
#include <stdarg.h>
#include <direct.h>
#include <stdlib.h>
#include <io.h>
#include <stdio.h>
#include <string.h>


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/

	/* Holds the current callback information */
  PDFINSTALLCALLBACK	l_pInstallCallback = NULL;
  void*			l_pInstallData = NULL;

	/* Holds information about the current installation process */
  dfinstallinfo_t	l_InstallInfo;

	/* Used when copying files */
  char l_DFCopyBuffer[DFINSTALL_COPYBUFFER_SIZE+1];

/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Function - int l_InstallCallback (void);
 *
 * Local function to call the current callback function, if any.
 *
 *=========================================================================*/
int l_InstallCallback (void) {
  if (l_pInstallCallback != NULL) return l_pInstallCallback(l_InstallInfo, l_pInstallData);
  return (0);
 }
/*===========================================================================
 *		End of Function l_InstallCallback()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - void l_SetDFInstallError (pString, ...);
 *
 *=========================================================================*/
void l_SetDFInstallError (const char* pString, ...) {
  va_list Args;

  va_start(Args, pString);
  ErrorHandler.AddErrorV(ERR_CUSTOM, 0, pString, Args);
  va_end(Args);
 }
/*===========================================================================
 *		End of Function l_SetDFInstallError()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool l_DFInstallCheckPath (const char* pPath);
 *
 * Returns true if the path exists.  Attempts to create the path if it 
 * exists.
 *
 *=========================================================================*/
bool l_DFInstallCheckPath (const char* pPath) {
  int Result;

	/* Ensure that the output path exists */
  Result = chdir(pPath);
  if (Result == 0) return(true);

	/* Attempt to create it */
  Result = mkdir(pPath);
  if (Result == 0) return (true);

  l_SetDFInstallError("Failed to create the directory '%s'!", pPath);
  return (false);
 }
/*===========================================================================
 *		End of Function l_DFInstallCheckPath()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool l_DFCopyFile (pSource, pDest);
 *
 *=========================================================================*/
bool l_DFCopyFile (const char* pSource, const char* pDest) {
  FILE*  pInputFile  = NULL;
  FILE*  pOutputFile = NULL;
  bool   ReturnResult = true;
  int    Result;
  long   lResult;
  long   CopiedBytes = 0;
  long   FileSize;
  long   ReadSize;  

	/* Update the installation information */
  l_InstallInfo.pFilename = pSource;
  l_InstallInfo.NumFiles++;
  l_InstallInfo.FileBytes = 0;

	/* Attempt to open source file for input */
  pInputFile = fopen(pSource, "rb");

  if (pInputFile == NULL) {
    l_SetDFInstallError("Failed to open the file '%s' for reading!", pSource);
    return (false);
   }

	/* Set the filesize */
  fseek(pInputFile, 0, SEEK_END);
  FileSize = ftell(pInputFile);
  l_InstallInfo.FileSize = FileSize;
  fseek(pInputFile, 0, SEEK_SET);

  	/* Attempt to open dest file for input */
  pOutputFile = fopen(pDest, "wb");

  if (pOutputFile == NULL) {
    l_SetDFInstallError("Failed to open the file '%s' for writing!", pDest);
    goto DFCOPYFILE_ERROR;
   }

	/* Update the callback */
  Result = l_InstallCallback();
  if (Result < 0) goto DFCOPYFILE_ERROR;

	/* Copy the file */
  while (CopiedBytes < FileSize) {
    ReadSize    = DFINSTALL_COPYBUFFER_SIZE;
    CopiedBytes += DFINSTALL_COPYBUFFER_SIZE;

    if (CopiedBytes > FileSize) {
      ReadSize -= CopiedBytes - FileSize;
      CopiedBytes = FileSize;
     }

    lResult = fread(l_DFCopyBuffer, 1, ReadSize, pInputFile);

    if (lResult != ReadSize) {
      l_SetDFInstallError("Failed to read %d byte!", ReadSize);
      goto DFCOPYFILE_ERROR;
     }

    lResult = fwrite(l_DFCopyBuffer, 1, ReadSize, pOutputFile);

    if (lResult != ReadSize) {
      l_SetDFInstallError("Failed to write %d byte!", ReadSize);
      goto DFCOPYFILE_ERROR;
     }

		/* Update the callback */
    l_InstallInfo.FileBytes += ReadSize;
    l_InstallInfo.NumBytes  += ReadSize;
    Result = l_InstallCallback();
    if (Result < 0) goto DFCOPYFILE_ERROR;
   }

 
DFCOPYFILE_END:
  if (pInputFile != NULL)  fclose(pInputFile);
  if (pOutputFile != NULL) fclose(pOutputFile);
  return (ReturnResult);

DFCOPYFILE_ERROR:
  ReturnResult = false;
  goto DFCOPYFILE_END;
 }
/*===========================================================================
 *		End of Function l_DFCopyFile()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool l_DFInstallPath (pSourcePath, pDestPath, pCDPath, pGamePath);
 *
 * Copies all filesin the source subpath to the same path in the destination
 * path.  Returns false on any error.
 *
 *=========================================================================*/
bool l_DFInstallPath (const char* pSourcePath, const char* pDestPath, const char* pCDPath, const char* pGamePath) {
  char		InputBuffer[_MAX_PATH+1];
  char		OutputBuffer[_MAX_PATH+1];
  bool		CopyResult;
  bool		ReturnResult = true;
  int		Result;
  long		FindHandle;
  _finddata_t	FindData;

	/* Create the input/output paths/filenames */
  _snprintf(InputBuffer,  _MAX_PATH, "%s%s*.*", pSourcePath, pCDPath);
  _snprintf(OutputBuffer, _MAX_PATH, "%s%s", pDestPath, pGamePath);

	/* Ensure a valid output path */
  Result = l_DFInstallCheckPath(OutputBuffer);
  if (!Result) return (false);

	/* Start finding and copying files */
  FindData.attrib = _A_NORMAL;
  FindHandle = _findfirst(InputBuffer, &FindData);
  Result = 0;

  while (Result != -1 && FindHandle != -1) {
    _snprintf(InputBuffer,  _MAX_PATH, "%s%s%s", pSourcePath, pCDPath, FindData.name);
    _snprintf(OutputBuffer, _MAX_PATH, "%s%s%s", pDestPath,   pGamePath, FindData.name);

    if ((FindData.attrib & _A_SUBDIR) == 0) {
      CopyResult = l_DFCopyFile(InputBuffer, OutputBuffer);

      if (!CopyResult) {
        ReturnResult = false;
        break;
       }
     }

    Result = _findnext(FindHandle, &FindData);
   }
  
  if (FindHandle != -1) _findclose(FindHandle);
  return (ReturnResult);
 }
/*===========================================================================
 *		End of Function l_DFInstallPath()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - int l_PackedCallback (PackInfo, pData);
 *
 * Called during the packed.dat extraction.
 *
 *=========================================================================*/
int l_PackedCallback (dfpackinfo_t& PackInfo, void* pData) {

	/* Update the filename at the start of the extraction */
  if (PackInfo.NumBytes == 0) {
    l_InstallInfo.pFilename = PackInfo.pFilename;
    l_InstallInfo.NumFiles++;
    l_InstallInfo.FileSize = PackInfo.FileSize;
    l_InstallInfo.NumBytes += PackInfo.FileSize;
   }

  l_InstallInfo.FileBytes = PackInfo.NumBytes;;

  return l_InstallCallback();
 }
/*===========================================================================
 *		End of Function l_PackedCallback()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool DFInstallCDGame (pSourcePath, pDestPath, pCallback, pData);
 *
 * Attempt to install the game from the CD path to the destination path.
 * returns false on any error.
 *
 *=========================================================================*/
bool DFInstallCDGame (const char* pSourcePath, const char* pDestPath,
			PDFINSTALLCALLBACK pCallback, void* pData) {
  bool Result;
  char InputBuffer[_MAX_PATH+1];
  char OutputBuffer[_MAX_PATH+1];

	/* Save the callback information */
  l_pInstallCallback = pCallback;
  l_pInstallData     = pData;

	/* Initialize the installation information */
  l_InstallInfo.FileBytes  = 0;
  l_InstallInfo.FileSize   = 0;
  l_InstallInfo.pFilename  = NULL;
  l_InstallInfo.NumBytes   = 0;
  l_InstallInfo.NumFiles   = 0;
  l_InstallInfo.TotalBytes = DFINSTALL_TOTAL_BYTES;
  l_InstallInfo.TotalFiles = DFINSTALL_TOTAL_FILES;

  Result = l_DFInstallPath(pSourcePath, pDestPath, "dagger\\", "");
  if (Result) Result = l_DFInstallPath(pSourcePath, pDestPath, "dagger\\arena2\\", "arena2\\");
  if (Result) Result = l_DFInstallPath(pSourcePath, pDestPath, "dagger\\arena2\\books\\", "arena2\\books\\");
  if (Result) Result = l_DFInstallPath(pSourcePath, pDestPath, "dagger\\arena2\\maps\\", "arena2\\maps\\");
  if (Result) Result = l_DFInstallPath(pSourcePath, pDestPath, "data\\", "data\\");

  	/* Extract files from the packed.dat file */
  if (Result) {
    _snprintf(InputBuffer,  _MAX_PATH, "%sdagger\\arena2\\packed.dat", pSourcePath);
    _snprintf(OutputBuffer, _MAX_PATH, "%sarena2\\", pDestPath);
    Result = DFPackExtract (InputBuffer, OutputBuffer, l_PackedCallback, NULL);
   }

	/* Finished installing */
  l_InstallInfo.FileBytes = 0;
  l_InstallInfo.pFilename = "";
  l_InstallCallback();
    	
  return (Result);
 }
/*===========================================================================
 *		End of Function DFInstallCDGame()
 *=========================================================================*/
 