/*===========================================================================
 *
 * File:	Dfpacked.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	January 17, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __DFPACKED_H
#define __DFPACKED_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "dl_err.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Constants used to extract a specific file from the PACKED.DAT */
  #define DFPACKRECORD_ARCH3D_BSA 0 
  #define DFPACKRECORD_DAGGER_SND 1

	/* Number of records(files) in the PACKED.DAT file */
  #define DFPACK_NUM_RECORDS 2

	/* Maximum length of the local error message string */
  #define DFPACK_ERRORMSG_SIZE 256

	/* Size of record header data to skip */
  #define DFPACK_RECORDHEADER_LENGTH 36

	/* Used when extracting the file */
  #define DFPACK_BUFFER_SIZE 1048576
  

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

	/* PACKED.DAT header structure */
  typedef struct {
	long	OffsetRecordDir;	/* Offset to record directory from start of file */
	long	OffsetOutputDir;	/* Offset to output directory from start of file */
  } dfpackheader_t;


	/* PACKED.DAT record structure */
  typedef struct {
	long	FileLength;		/* Length of decompressed file */
	long	Unknown;		/* Always zero */
	char	Filename[13];		/* Filename, including NULL terminator */
	long	Offset;			/* Offset to record from start of file */
  } dfpackrecord_t;

	/* Type used for extraction callback */
  typedef struct {
	const char*	pFilename;	/* Currently extracting... */
	long		FileSize;	/* Total size of file */
	long		NumBytes;	/* Number of bytes currently extracted */
   } dfpackinfo_t;

	/* Callback type */
  typedef int (*PDFPACKCALLBACK) (dfpackinfo_t& PaclInfo, void* pData);

#pragma pack(pop)
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Function Prototypes
 *
 *=========================================================================*/

	/* Main access function for extracting files from the PACKED.DAT */
  bool DFPackExtract (const char* pInputFile, const char* pOutputPath, 
			PDFPACKCALLBACK pCallback, void *pData);

/*===========================================================================
 *		End of Function Prototypes
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Dfpacked.H
 *=========================================================================*/
