/*===========================================================================
 *
 * File:	NifBSPArrayController.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Tuesday, December 24, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "NifBSPArrayController.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifBSPArrayController.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBSPACRecord Method - bool Read (FileHandle);
 *
 * Description
 *
 *=========================================================================*/
bool CNifBSPACRecord::Read (CGenFile& FileHandle) {
  bool Result;

  Destroy();

  //SystemLog.Printf ("\t\t\tBSPACRecord = 0x%08lX", FileHandle.Tell());

  Result = FileHandle.Read((char *)&m_Data, sizeof(nifbspacrecord_t));
  if (Result) Result = FileHandle.ReadShort(m_Unknown1);
  if (Result) Result = FileHandle.ReadShort(m_Index);
  if (!Result) return (false);

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBSPACRecord::Read()
 *=========================================================================*


/*===========================================================================
 *
 * Class CNifBSPACRecord Method - bool Write (FileHandle);
 *
 * Description
 *
 *=========================================================================*/
bool CNifBSPACRecord::Write (CGenFile& FileHandle) {
  bool Result;

  Result = FileHandle.Write((char *)&m_Data, sizeof(nifbspacrecord_t));
  if (Result) Result = FileHandle.WriteShort(m_Unknown1);
  if (Result) Result = FileHandle.WriteShort(m_Index);
  if (!Result) return (false);

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBSPACRecord::Write()
 *=========================================================================*


/*===========================================================================
 *
 * Class CNifBSPArrayController Constructor
 *
 *=========================================================================*/
CNifBSPArrayController::CNifBSPArrayController () {
  //DEFINE_FUNCTION("CNifBSPArrayController::CNifBSPArrayController()");
  m_pRecords = NULL;
  m_NumRecords1 = 0;
  m_NumRecords2 = 0;
 }
/*===========================================================================
 *		End of Class CNifBSPArrayController Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBSPArrayController Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifBSPArrayController::Destroy (void) {
  DEFINE_FUNCTION("CNifBSPArrayController::Destroy()");
  m_NumRecords1 = 0;
  m_NumRecords2 = 0;
  DestroyArrayPointer(m_pRecords);

  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifBSPArrayController::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBSPArrayController Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifBSPArrayController::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifBSPArrayController::Read()");
  bool Result;
  int  Index;

  Destroy();

  Result = FileHandle.ReadLong(m_Unknown1);
  if (Result) Result = FileHandle.ReadShort(m_Unknown2);
  if (Result) Result = FileHandle.Read((char *)&m_Position1, sizeof(nifpos4_t));
  if (Result) Result = FileHandle.ReadLong(m_Unknown3);
  if (Result) Result = FileHandle.Read((char *)&m_Floats1, sizeof(float)*NIF_BSPAC_NUMFLOATS1);
  if (Result) Result = FileHandle.ReadChar(m_Unknown4);
  if (Result) Result = FileHandle.Read((char *)&m_Position2, sizeof(nifpos_t));
  if (Result) Result = FileHandle.ReadLong(m_Unknown5);
  if (Result) Result = FileHandle.Read((char *)&m_Floats2, sizeof(float)*NIF_BSPAC_NUMFLOATS2);
  if (Result) Result = FileHandle.ReadLong(m_Unknown6);
  if (Result) Result = FileHandle.ReadLong(m_Unknown7);
  if (Result) Result = FileHandle.ReadShort(m_Unknown8);
  if (Result) Result = FileHandle.ReadShort(m_NumRecords1);
  if (Result) Result = FileHandle.ReadShort(m_NumRecords2);
  
  if (GetNumRecords() > 0) {
    CreateArrayPointer(m_pRecords, CNifBSPACRecord, GetNumRecords());

    for (Index = 0; Index < GetNumRecords(); Index++) {
      Result = m_pRecords[Index].Read(FileHandle);
      if (!Result) return (false);
     }
   }

  //SystemLog.Printf ("\t\tAfterRecords = 0x%08lX", FileHandle.Tell());

  Result = FileHandle.ReadLong(m_Unknown9);
  if (Result) Result = FileHandle.ReadLong(m_Unknown10);
  if (Result) Result = FileHandle.ReadLong(m_Unknown11);
  if (Result) Result = FileHandle.ReadChar(m_Unknown12);
  if (!Result) return (false);

  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBSPArrayController::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBSPArrayController Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifBSPArrayController::Write (CGenFile& FileHandle) {
  //DEFINE_FUNCTION("CNifBSPArrayController::Write()");
  bool Result;
  int  Index;

	/* Call the base class record method */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

  Result = FileHandle.WriteLong(m_Unknown1);
  if (Result) Result = FileHandle.WriteShort(m_Unknown2);
  if (Result) Result = FileHandle.Write((char *)&m_Position1, sizeof(nifpos4_t));
  if (Result) Result = FileHandle.WriteLong(m_Unknown3);
  if (Result) Result = FileHandle.Write((char *)&m_Floats1, sizeof(float)*NIF_BSPAC_NUMFLOATS1);
  if (Result) Result = FileHandle.WriteChar(m_Unknown4);
  if (Result) Result = FileHandle.Write((char *)&m_Position2, sizeof(nifpos_t));
  if (Result) Result = FileHandle.WriteLong(m_Unknown5);
  if (Result) Result = FileHandle.Write((char *)&m_Floats2, sizeof(float)*NIF_BSPAC_NUMFLOATS2);
  if (Result) Result = FileHandle.WriteLong(m_Unknown6);
  if (Result) Result = FileHandle.WriteLong(m_Unknown7);
  if (Result) Result = FileHandle.WriteShort(m_Unknown8);
  if (Result) Result = FileHandle.WriteShort(m_NumRecords1);
  if (Result) Result = FileHandle.WriteShort(m_NumRecords2);
  if (!Result) return (false);

  if (GetNumRecords() > 0) {
    for (Index = 0; Index < GetNumRecords(); Index++) {
      Result = m_pRecords[Index].Write(FileHandle);
      if (!Result) return (false);
     }
   }

  Result = FileHandle.WriteLong(m_Unknown9);
  if (Result) Result = FileHandle.WriteLong(m_Unknown10);
  if (Result) Result = FileHandle.WriteLong(m_Unknown11);
  if (Result) Result = FileHandle.WriteChar(m_Unknown12);
  if (!Result) return (false);

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBSPArrayController::Write()
 *=========================================================================*/