/*===========================================================================
 *
 * File:	NifBSAnimationNode.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "NifBSAnimationNode.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifBSAnimationNode.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBSAnimationNode Constructor
 *
 *=========================================================================*/
CNifBSAnimationNode::CNifBSAnimationNode () {
  //DEFINE_FUNCTION("CNifBSAnimationNode::CNifBSAnimationNode()");
 
 }
/*===========================================================================
 *		End of Class CNifBSAnimationNode Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBSAnimationNode Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifBSAnimationNode::Destroy (void) {
  //DEFINE_FUNCTION("CNifBSAnimationNode::Destroy()");
  m_Array1.Destroy();
  m_Array2.Destroy();
  m_Array3.Destroy();
  m_Array4.Destroy();

  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifBSAnimationNode::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBSAnimationNode Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifBSAnimationNode::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifBSAnimationNode::Read()");
  bool Result;
  
	/* Delete the current record contents */ 
  Destroy();

	/* Output the node name */
  Result = NifReadString(FileHandle, m_NodeName);
  if (!Result) return (false);
  //SystemLog.Printf ("\t\tNodeOffset = 0x%08lX", FileHandle.Tell());

  	/* Input first three numbers */
  Result = FileHandle.ReadLong(m_Unknown1);
  if (Result) Result = FileHandle.ReadLong(m_Unknown2);
  if (Result) Result = FileHandle.ReadShort(m_Unknown3);
  if (!Result) return (false);
  //SystemLog.Printf ("\t\tNodeOffset = 0x%08lX", FileHandle.Tell());

	/* Input coordinates */
  Result = FileHandle.Read((char *)&m_Coordinates[0], sizeof(float)*NIF_BSAN_NUMCOORS);
  if (!Result) return (false);
  //SystemLog.Printf ("\t\tNodeOffset = 0x%08lX", FileHandle.Tell());

  Result = m_Array1.Read(FileHandle);
  if (Result) Result = m_Array2.Read(FileHandle);
  if (Result) Result = m_Array3.Read(FileHandle);
  if (Result) Result = m_Array4.Read(FileHandle);
  if (!Result) return (false);
  //SystemLog.Printf ("\t\tNodeOffset = 0x%08lX", FileHandle.Tell());

	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBSAnimationNode::Read()
 *=========================================================================*/
  

/*===========================================================================
 *
 * Class CNifBSAnimationNode Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifBSAnimationNode::Write (CGenFile& FileHandle) {
  bool Result;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

	/* Output the node name */
  Result = NifWriteString(FileHandle, m_NodeName);
  if (!Result) return (false);

  	/* Output first three numbers */
  Result = FileHandle.WriteLong(m_Unknown1);
  if (Result) Result = FileHandle.WriteLong(m_Unknown2);
  if (Result) Result = FileHandle.WriteShort(m_Unknown3);
  if (!Result) return (false);

	/* Output coordinates */
  Result = FileHandle.Write((char *)&m_Coordinates[0], sizeof(float)*NIF_BSAN_NUMCOORS);
  if (!Result) return (false);

  	/* Output indices */
  Result = m_Array1.Write(FileHandle);
  if (Result) Result = m_Array2.Write(FileHandle);
  if (Result) Result = m_Array3.Write(FileHandle);
  if (Result) Result = m_Array4.Write(FileHandle);
  if (!Result) return (false);

	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBSAnimationNode::Write()
 *=========================================================================*/



