/*===========================================================================
 *
 * File:	Niffile.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "Stdafx.h"
#include "niffile.h"
#include "NifTriShape.h"
#include "NifTriShapeData.h"
#include "NifMaterialProperty.h"
#include "NifStringData.h"
#include "NifParticleNode.h"
#include "NifTextureProperty.h"
#include "NifSourceTexture.h"
#include "NifEndFile.h"
#include "NifRootCollisonNode.h"
#include "NifAlphaProperty.h"
#include "NifAmbientLight.h"
#include "NifVertexColorProperty.h"
#include "niftextkeyextradata.h"
#include "NifKeyFrameController.h"
#include "NifKeyframeData.h"
#include "NifVisController.h"
#include "NifVisData.h"
#include "NifZBufferProperty.h"
#include "NifRotatingParticles.h"
#include "NifParticleSysController.h"
#include "NifParticleGrowFade.h"
#include "NifRotatingParticlesData.h"
#include "NifAutoNormalParticlesData.h"
#include "NifBSAnimationNode.h"
#include "NifUVController.h"
#include "NifUVData.h"
#include "NifTextureEffect.h"
#include "NifGravity.h"
#include "NifWireframeProperty.h"
#include "NifSkinInstance.h"
#include "NifSkinData.h"
#include "NifBSPArrayController.h"
#include "NifBillboardNode.h"
#include "NifFloatData.h"
#include "NifParticleColorModifier.h"
#include "NifColorData.h"
#include "NifVertWeightsExtraData.h"
#include "NifGeomMorpherController.h"
#include "NifCamera.h"
#include "NifParticleRotation.h"
#include "NifMorphData.h"
#include "NifPathController.h"
#include "NifPosData.h"
#include "NifPixelData.h"
#include "NifAutoNormalParticles.h"
#include "NifShadeProperty.h"
#include "NifSpecularProperty.h"
#include "NifDirectionalLight.h"
#include "NifEof.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifFile.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Constructor
 *
 *=========================================================================*/
CNifFile::CNifFile () {
  //DEFINE_FUNCTION("CNifFile::CNifFile()");

  m_ReadHeader = false;
 }
/*===========================================================================
 *		End of Class CNifFile Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifFile::Destroy (void) {
  //DEFINE_FUNCTION("CNifFile::Destroy()");

  DeleteContents();

	/* Call the base class method */
  CGenFile::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifFile::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - void DeleteContents (void);
 *
 *=========================================================================*/
void CNifFile::DeleteContents (void) {
  DEFINE_FUNCTION("CNifFile::DeleteContents()");
  CNifBaseRecord* pRecord;
  m_ReadHeader = false;

	/* Delete all records in the list */
  while (!m_Records.IsEmpty()) {
    pRecord = (CNifBaseRecord *) m_Records.RemoveHead();
    DestroyPointer(pRecord);
   }

 }
/*===========================================================================
 *		End of Class Method CNifFile::DeleteContents()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - CNifBaseRecord* CreateRecord (RecordName, Length);
 *
 * Protected class method to create a new NIF record based on the input
 * record name.  Returns NULL on any error.
 *
 *=========================================================================*/
CNifBaseRecord* CNifFile::CreateRecord (const CString& RecordName, const long& Length) {
  DEFINE_FUNCTION("CNifFile::CreateRecord()");
  CNifBaseRecord* pRecord = NULL;

  if (RecordName.CompareNoCase("NiNode") == 0) {
    CreatePointer(pRecord, CNifNode);
   }
  else if (RecordName.CompareNoCase("NiTriShape") == 0) {
    CreatePointer(pRecord, CNifTriShape);
   }
  else if (RecordName.CompareNoCase("NiAmbientLight") == 0) {
    CreatePointer(pRecord, CNifAmbientLight);
   }
  else if (RecordName.CompareNoCase("NiTriShapeData") == 0) {
    CreatePointer(pRecord, CNifTriShapeData);
   }
  else if (RecordName.CompareNoCase("NiMaterialProperty") == 0) {
    CreatePointer(pRecord, CNifMaterialProperty);
   }
  else if (RecordName.CompareNoCase("NiStringExtraData") == 0) {
    CreatePointer(pRecord, CNifStringData);
   }
  else if (RecordName.CompareNoCase("NiBSParticleNode") == 0) {
    CreatePointer(pRecord, CNifParticleNode);
   }
  else if (RecordName.CompareNoCase("NiTexturingProperty") == 0) {
    CreatePointer(pRecord, CNifTexturingProperty);
   }
  else if (RecordName.CompareNoCase("NiSourceTexture") == 0) {
    CreatePointer(pRecord, CNifSourceTexture);
   }
  else if (RecordName.CompareNoCase("RootCollisionNode") == 0) {
    CreatePointer(pRecord, CNifRootCollisionNode);
   }
  else if (RecordName.CompareNoCase("AvoidNode") == 0) {
    CreatePointer(pRecord, CNifRootCollisionNode);
   }
  else if (RecordName.CompareNoCase("NiAlphaProperty") == 0) {
    CreatePointer(pRecord, CNifAlphaProperty);
   }
  else if (RecordName.CompareNoCase("NiVertexColorProperty") == 0) {
    CreatePointer(pRecord, CNifVertexColorProperty);
   }
  else if (RecordName.CompareNoCase("NiTextKeyExtraData") == 0) {
    CreatePointer(pRecord, CNifTextKeyExtraData);
   }
  else if (RecordName.CompareNoCase("NiKeyframeController") == 0) {
    CreatePointer(pRecord, CNifKeyFrameController);
   }
  else if (RecordName.CompareNoCase("NiMaterialColorController") == 0) {
    CreatePointer(pRecord, CNifKeyFrameController);
   }
  else if (RecordName.CompareNoCase("NiGeomMorpherController") == 0) {
    CreatePointer(pRecord, CNifGeomMorpherController);
   }
  else if (RecordName.CompareNoCase("NiKeyframeData") == 0) {
    CreatePointer(pRecord, CNifKeyFrameData);
   }
  else if (RecordName.CompareNoCase("NiVisController") == 0) {
    CreatePointer(pRecord, CNifVisController);
   }
  else if (RecordName.CompareNoCase("NiAlphaController") == 0) {
    CreatePointer(pRecord, CNifVisController);
   }
  else if (RecordName.CompareNoCase("NiUVController") == 0) {
    CreatePointer(pRecord, CNifUVController);
   }
  else if (RecordName.CompareNoCase("NiVisData") == 0) {
    CreatePointer(pRecord, CNifVisData);
   }
  else if (RecordName.CompareNoCase("NiPosData") == 0) {
    CreatePointer(pRecord, CNifPosData);
   }
  else if (RecordName.CompareNoCase("NiZBufferProperty") == 0) {
    CreatePointer(pRecord, CNifZBufferProperty);
   }
  else if (RecordName.CompareNoCase("NiRotatingParticles") == 0) {
    CreatePointer(pRecord, CNifRotatingParticles);
   }
  else if (RecordName.CompareNoCase("NiAutoNormalParticles") == 0) {
    CreatePointer(pRecord, CNifAutoNormalParticles);
   }
  else if (RecordName.CompareNoCase("NiRotatingParticlesData") == 0) {
    CreatePointer(pRecord, CNifRotatingParticlesData);
   }
  else if (RecordName.CompareNoCase("NiAutoNormalParticlesData") == 0) {
    CreatePointer(pRecord, CNifAutoNormalParticlesData);
   }
  else if (RecordName.CompareNoCase("NiParticleSystemController") == 0) {
    CreatePointer(pRecord, CNifParticleSystemController);
   }
  else if (RecordName.CompareNoCase("NiParticleGrowFade") == 0) {
    CreatePointer(pRecord, CNifParticleGrowFade);
   }
  else if (RecordName.CompareNoCase("NiCamera") == 0) {
    CreatePointer(pRecord, CNifCamera);
   }
  else if (RecordName.CompareNoCase("NiBSAnimationNode") == 0) {
    CreatePointer(pRecord, CNifBSAnimationNode);
   }
  else if (RecordName.CompareNoCase("NiUVData") == 0) {
    CreatePointer(pRecord, CNifUVData);
   }
  else if (RecordName.CompareNoCase("NiTextureEffect") == 0) {
    CreatePointer(pRecord, CNifTextureEffect);
   }
  else if (RecordName.CompareNoCase("NiGravity") == 0) {
    CreatePointer(pRecord, CNifGravity);
   }
  else if (RecordName.CompareNoCase("NiWireframeProperty") == 0) {
    CreatePointer(pRecord, CNifWireframeProperty);
   }
  else if (RecordName.CompareNoCase("NiSkinInstance") == 0) {
    CreatePointer(pRecord, CNifSkinInstance);
   }
  else if (RecordName.CompareNoCase("NiSkinData") == 0) {
    CreatePointer(pRecord, CNifSkinData);
   }
  else if (RecordName.CompareNoCase("NiBSPArrayController") == 0) {
    CreatePointer(pRecord, CNifBSPArrayController);
   }
  else if (RecordName.CompareNoCase("NiBillboardNode") == 0) {
    CreatePointer(pRecord, CNifBillboardNode);
   }
  else if (RecordName.CompareNoCase("NiFloatData") == 0) {
    CreatePointer(pRecord, CNifFloatData);
   }
  else if (RecordName.CompareNoCase("NiParticleColorModifier") == 0) {
    CreatePointer(pRecord, CNifParticleColorModifier);
   }
  else if (RecordName.CompareNoCase("NiColorData") == 0) {
    CreatePointer(pRecord, CNifColorData);
   }
  else if (RecordName.CompareNoCase("NiVertWeightsExtraData") == 0) {
    CreatePointer(pRecord, CNifVertWeightsExtraData);
   }
  else if (RecordName.CompareNoCase("NiParticleRotation") == 0) {
    CreatePointer(pRecord, CNifParticleRotation);
   }
  else if (RecordName.CompareNoCase("NiMorphData") == 0) {
    CreatePointer(pRecord, CNifMorphData);
   }
  else if (RecordName.CompareNoCase("NiPathController") == 0) {
    CreatePointer(pRecord, CNifPathController);
   }
  else if (RecordName.CompareNoCase("NiPixelData") == 0) {
    CreatePointer(pRecord, CNifPixelData);
   }
  else if (RecordName.CompareNoCase("NiShadeProperty") == 0) {
    CreatePointer(pRecord, CNifShadeProperty);
   }
  else if (RecordName.CompareNoCase("NiSpecularProperty") == 0) {
    CreatePointer(pRecord, CNifSpecularProperty);
   }
  else if (RecordName.CompareNoCase("NiDirectionalLight") == 0) {
    CreatePointer(pRecord, CNifDirectionalLight);
   }
  else if (RecordName.IsEmpty() || Length == 1 || Length == 2) {
    //CreatePointer(pRecord, CNifEndFile);
    CreatePointer(pRecord, CNifEOF);
   }
  else { 
    ErrorHandler.AddError(ERR_BADINPUT, "Unknown NIF record type '%s' (offset 0x%08lX)!", RecordName, Tell());
   }

  if (pRecord != NULL) {
    //pRecord->SetName(RecordName);
    SystemLog.Printf ("Found %s (0x%08lX)", RecordName, Tell());
    //SystemLog.IncrementTabs();
   }

  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CNifFile::CreateRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - CNifBaseRecord* FindFirstRecord (Pos, pName);
 *
 * Finds the first NIF record that matches the given record name.  Is not case
 * sensitive.  Returns NULL on any error.
 *
 *=========================================================================*/
CNifBaseRecord* CNifFile::FindFirstRecord (POSITION& Pos, const TCHAR* pName) {

	/* Check for an empty list */
  if (m_Records.IsEmpty()) {
    Pos = NULL;
    return (NULL);
   }

  Pos = m_Records.GetHeadPosition();
  return FindNextRecord(Pos, pName);
 }
/*===========================================================================
 *		End of Class Method CNifFile::FindFirstRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - CNifBaseRecord* FindNextRecord (Pos, pName);
 *
 * Finds the next record in the NIF file starting at the next record from the
 * given array index (not including the given position).  Returns NULL
 * on any error.  Is not case sensitive.  The input
 * position is updated to point to the found record, or NULL if nothing was
 * found.
 *
 *=========================================================================*/
CNifBaseRecord* CNifFile::FindNextRecord (POSITION& Pos, const TCHAR* pName) {
  CNifBaseRecord* pRecord;

	/* Check for an empty list */
  if (m_Records.IsEmpty()) {
    Pos = NULL;
    return (NULL);
   }

	/* Get the first record to be searched */
  if (Pos == NULL) return (NULL);

	/* Get the next record */
  pRecord = (CNifBaseRecord *) m_Records.GetNext(Pos);

	/* Search until the end of the list or a match is found */
  while (Pos != NULL) {
    if (pRecord->IsName(pName)) return (pRecord);
    pRecord = (CNifBaseRecord *) m_Records.GetNext(Pos);
   }

	/* Nothing found */
  return (NULL);
 }
/*===========================================================================
 *		End of Class Method CNifFile::FindNextRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - CNifBaseRecord* GetFirstRecord (Pos);
 *
 * Returns the first record in the NIF file.  Returns NULL
 * on any error.  
 *
 *=========================================================================*/
CNifBaseRecord* CNifFile::GetFirstRecord (POSITION& Pos) {
  CNifBaseRecord* pRecord;

	/* Check for an empty list */
  if (m_Records.IsEmpty()) {
    Pos = NULL;
    return (NULL);
   }

  Pos = m_Records.GetHeadPosition();
  if (Pos != NULL) m_Records.GetNext(Pos);
  pRecord = (CNifBaseRecord *) m_Records.GetHead();

  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CNifFile::GetFirstRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - CNifBaseRecord* GetNextRecord (Pos);
 *
 * Finds the next record in the NIF file starting at the next record from the
 * given array index (not including the given position).  Returns NULL
 * on any error.  
 *
 *=========================================================================*/
CNifBaseRecord* CNifFile::GetNextRecord (POSITION& Pos) {
  CNifBaseRecord* pRecord;

	/* Check for an empty list */
  if (m_Records.IsEmpty()) {
    Pos = NULL;
    return (NULL);
   }

	/* Ignore if input position is not valid */
  if (Pos == NULL) return (NULL);

  pRecord = (CNifBaseRecord *) m_Records.GetNext(Pos);
  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CNifFile::GetNextRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - CNifBaseRecord* GetRecord (Index);
 *
 * Description
 *
 *=========================================================================*/
CNifBaseRecord* CNifFile::GetRecord (const int Index) {
  POSITION Pos;

  Pos = m_Records.FindIndex(Index);
  if (Pos == NULL) return (NULL);

  return (CNifBaseRecord *) m_Records.GetAt(Pos);
 }
/*===========================================================================
 *		End of Class Method CNifFile::GetRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - void ModNodePos (Flags, Delta);
 *
 * Modifies all the node position data by the given amount.  The Flags are
 * a combination of 
 *			NIF_MODX
 *			NIF_MODY
 *			NIF_MODZ
 *
 * Similarily with the TriShape and TriShapeData record methods.
 *
 *=========================================================================*/
void CNifFile::ModNodePos (const int Flags, const float Delta) {
  CNifBaseRecord*	pRecord;
  CNifNode*		pNode;
  POSITION		RecordPos;

  pRecord = FindFirstRecord(RecordPos, _T("NiNode"));

  while (pRecord != NULL) {
    pNode = (CNifNode *) pRecord;
    pNode->ModPos(Flags, Delta);
    pRecord = FindNextRecord(RecordPos, _T("NiNode"));
   }
 }


void CNifFile::ModNodePosWorld (const int Flags, const float Delta) {
  CNifBaseRecord*	pRecord;
  CNifNode*		pNode;
  POSITION		RecordPos;

  pRecord = FindFirstRecord(RecordPos, _T("NiNode"));

  while (pRecord != NULL) {
    pNode = (CNifNode *) pRecord;
    pNode->ModPosWorld(Flags, Delta);
    pRecord = FindNextRecord(RecordPos, _T("NiNode"));
   }
 }



void CNifFile::ModTriShapePos (const int Flags, const float Delta) {
  CNifBaseRecord*	pRecord;
  CNifTriShape*		pTriShape;
  POSITION		RecordPos;

  pRecord = FindFirstRecord(RecordPos, _T("NiTriShape"));

  while (pRecord != NULL) {
    pTriShape = (CNifTriShape *) pRecord;
    pTriShape->ModPos(Flags, Delta);
    pRecord = FindNextRecord(RecordPos, _T("NiTriShape"));
   }
 }


void CNifFile::ModTriShapeDataPos (const int Flags, const float Delta) {
  CNifBaseRecord*	pRecord;
  CNifTriShapeData*	pTriShape;
  POSITION		RecordPos;

  pRecord = FindFirstRecord(RecordPos, _T("NiTriShapeData"));

  while (pRecord != NULL) {
    pTriShape = (CNifTriShapeData *) pRecord;
    pTriShape->ModPos(Flags, Delta);
    pRecord = FindNextRecord(RecordPos, _T("NiTriShapeData"));
   }
 }


void CNifFile::ModTriShapeScale (const float Delta) { 
  CNifBaseRecord*	pRecord;
  CNifTriShape*		pTriShape;
  POSITION		RecordPos;

  pRecord = FindFirstRecord(RecordPos, _T("NiTriShape"));

  while (pRecord != NULL) {
    pTriShape = (CNifTriShape *) pRecord;
    pTriShape->ModScale(Delta);
    pRecord = FindNextRecord(RecordPos, _T("NiTriShape"));
   }
 }
/*===========================================================================
 *		End of Class Method CNifFile::ModNodePos()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - bool Read (pFilename);
 *
 * Public method to open and read the given NIF file.  Returns false on 
 * any error.
 *
 *=========================================================================*/
bool CNifFile::Read (const char* pFilename) {
  DEFINE_FUNCTION("CNifFile::Read()");
  bool		  Result;
    
	/* Attempt to open the file for input */
  Result = Open(pFilename, "rb");
  if (!Result) return (false);

  Result = Read();
  Close();

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CNifFile::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - bool Read (void);
 *
 * Protected class method whichs attempts to input an open NIF file.  Returns
 * false on any error.
 *
 *=========================================================================*/
bool CNifFile::Read (void) {
  DEFINE_FUNCTION("CNifFile::Read()");
  bool		  Result;
  CNifBaseRecord* pRecord;
  CString	  Buffer;
  long		  Offset;
  long		  FileSize;
  long		  Length;

	/* Delete the current contents of the file */
  DeleteContents();

  Result = GetFileSize(FileSize);
  if (!Result) return (false);

	/* Attempt to input the file header */
  Result = ReadHeader();
  if (!Result) return (false);

	/* Read in entire file */
  while (!IsEOF() && Tell() < FileSize) {
    Offset = Tell();

		/* Input the record name string */
    Result = NifReadString(*this, Buffer, &Length);
    if (!Result) return (false);

    Offset = Tell();

		/* Create the appropriate record */
    pRecord = CreateRecord(Buffer, Length);
    if (pRecord == NULL) return (false);
    m_Records.AddTail(pRecord);

		/* Attempt to input the record */
    pRecord->SetName(Buffer);
    pRecord->SetNameLength(Length);

    Result = pRecord->Read(*this);

    pRecord->SetStartOffset(Offset);
    pRecord->SetEndOffset(Tell());
    //SystemLog.Printf ("End Offset = 0x%08lX", Tell());
    //SystemLog.DecrementTabs();
    if (!Result) return (false);
   
   }

  if (Tell() != FileSize) {
    SystemLog.Printf ("Possible under/over read of file (%d bytes)", Tell() - FileSize);
    return (false);
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifFile::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - bool ReadHeader (void);
 *
 * Protected class method to input the NIF file header.  Returns false on
 * any error.  Resets the file pointer to the start of the file before
 * reading.
 *
 *=========================================================================*/
bool CNifFile::ReadHeader (void) {
  DEFINE_FUNCTION("CNifFile::ReadHeader()");
  bool Result;

  m_ReadHeader = false;

	/* Move file pointer to beginning of file */
  Result = Seek(0, SEEK_SET);
  if (!Result) return (false);

	/* Attempt to input header */
  Result = CGenFile::Read((char *)&m_Header, sizeof(nifheader_t));
  if (!Result) return (false);

  m_ReadHeader = true;
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifFile::ReadHeader()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - bool Write (pFilename);
 *
 * Public class method whichs attempts to output a NIF file with the given
 * name.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifFile::Write (const char* pFilename) {
  DEFINE_FUNCTION("CNifFile::Write()");
  bool		  Result;
  
	/* Attempt to open the file for output */
  Result = Open(pFilename, "wb");
  if (!Result) return (false);

	/* Attempt to output the file */
  Result = Write();
  Close();

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CNifFile::Write()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - bool Write (void);
 *
 * Protected class method whichs attempts to output an open NIF file.  Returns
 * false on any error.
 *
 *=========================================================================*/
bool CNifFile::Write (void) {
  DEFINE_FUNCTION("CNifFile::Write()");
  bool		  Result;
  POSITION	  RecordPos;
  CNifBaseRecord* pRecord;

	/* Attempt to output the file header */
  Result = WriteHeader();
  if (!Result) return (false);

	/* Output all records in the list */
  RecordPos = m_Records.GetHeadPosition();

  while (RecordPos != NULL) {
    pRecord = (CNifBaseRecord *) m_Records.GetNext(RecordPos);

    pRecord->SetStartOffset(Tell());
    Result = pRecord->Write(*this);
    pRecord->SetEndOffset(Tell());

    if (!Result) return (false);
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifFile::Write()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFile Method - bool WriteHeader (void);
 *
 * Protected class method to output the NIF file header.  Returns false on
 * any error.  Resets the file pointer to the start of the file before
 * reading.
 *
 *=========================================================================*/
bool CNifFile::WriteHeader (void) {
  DEFINE_FUNCTION("CNifFile::WriteHeader()");
  bool Result;

	/* Ensure valid object state */
  ASSERT(IsOpen());
  if (!m_ReadHeader) return (false);

	/* Attempt to output header */
  Result = CGenFile::Write((char *)&m_Header, sizeof(nifheader_t));
  if (!Result) return (false);

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifFile::WriteHeader()
 *=========================================================================*/
  