/*===========================================================================
 *
 * File:	NifKeyFrameData.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Monday, December 23, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __NIFKEYFRAMEDATA_H
#define __NIFKEYFRAMEDATA_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "NifBase.h"
  #include "NifKeyFrameTypes.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/
  
  #define NIF_KEYFRAME_NUMVALUES 48

	/* Basic types of arrays */
  #define NIF_KF_POSITION	2
  #define NIF_KF_SCALE		3
  #define NIF_KF_ROTATION	1

	/* Main types of controllers */
  #define NIF_KFDATA_LINEAR	0x0001
  #define NIF_KFDATA_BEZIER	0x0002
  #define NIF_KFDATA_TCB	0x0003
  #define NIF_KFDATA_EULER	0x0004
  #define NIF_KFDATA_UNKNOWN	0x0005

	/* Combination of controllers and types */
  #define NIF_KFROT_LINEAR	0x10001
  #define NIF_KFROT_BEZIER	0x10002
  #define NIF_KFROT_TCB		0x10003
  #define NIF_KFROT_EULER	0x10004
  #define NIF_KFROT_UNKNOWN	0x10005

  #define NIF_KFPOS_LINEAR	0x20001
  #define NIF_KFPOS_BEZIER	0x20002
  #define NIF_KFPOS_TCB		0x20003
  
  #define NIF_KFSCA_LINEAR	0x30001
  #define NIF_KFSCA_BEZIER	0x30002
  #define NIF_KFSCA_TCB		0x30003

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

  struct nifkfposlin_t {
	float    Time;
	nifpos_t Position;
   };

  struct nifkfposbez_t {
	float	 Time;
	nifpos_t Position;
	nifpos_t InBezier;
	nifpos_t OutBezier;
   };

  struct nifkfpostcb_t {
	float		Time;
	nifpos_t	Position;
	float		Tension;
	float		Continuity;
	float		Bias;
   };
   			
  struct nifkfscalin_t {
	float	Time;
	float	Scale;
   };


  struct nifkfscabez_t {
	float	Time;
	float	Scale;
	float	InBezier;
	float	OutBezier;
   };

  struct nifkfscatcb_t {
	float	Time;
	float   Scale;
	float	Tension;
	float	Continuity;
	float	Bias;
   };

  struct nifkfrotlin_t {
	float	Time;
	float	Value1;
	float	Value2;
	float	Value3;
	float	Value4;
   };
   
  struct nifkfrottcb_t {
	float	Time;
	float	CosAngle2;
	float	SinAngleX;
	float	SinAngleY;
	float	SinAngleZ;
	float	Tension;
	float	Continuity;
	float	Bias;
    };

  struct nifkfroteul_t {
	byte	Data[16];	/* Unknown format */
    };
   
   struct nifkfrotunk_t {
	byte	Data[32];
    };

#pragma pack(pop)
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifKeyFrameDataArray Definition
 *
 * Description
 *
 *=========================================================================*/
class CNifKeyFrameDataArray {

  /*---------- Begin Private Class Members ----------------------*/
private:
  long   m_Size;
  long	 m_ControlType;
  float* m_pData;
  long   m_RecordSize;		/* Number of floats per record */
  int	 m_Type;		/* Which type of array, pos/sca/rot *
  

  /*---------- Begin Protected Class Methods --------------------*/
protected:
	
	/* Returns the record size for the given type of controller */
  long FindRecordSize (const int ControlType);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CNifKeyFrameDataArray();
  virtual ~CNifKeyFrameDataArray() { Destroy(); }
  virtual void Destroy (void);

	/* Get class members */
  int  GetSize        (void) const { return (m_Size); }
  bool IsEmpty        (void) const { return (m_Size == 0); }
  bool IsPosArray     (void) const { return (m_Type == NIF_KF_POSITION); }
  bool IsScaArray     (void) const { return (m_Type == NIF_KF_SCALE); }
  bool IsRotArray     (void) const { return (m_Type == NIF_KF_ROTATION); }
  bool IsLinData      (void) const { return (m_ControlType == NIF_KFDATA_LINEAR); }
  bool IsTcbData      (void) const { return (m_ControlType == NIF_KFDATA_TCB); }
  bool IsBezData      (void) const { return (m_ControlType == NIF_KFDATA_BEZIER); }
  bool IsEulData      (void) const { return (m_ControlType == NIF_KFDATA_EULER); }
  bool IsUnkData      (void) const { return (m_ControlType == NIF_KFDATA_UNKNOWN); }
  bool IsValidKey     (const int Index) const { return (Index >= 0 && Index < m_Size); }
  bool CheckKey       (const int Index);
  int  GetType        (void) const { return (m_Type); }
  int  GetControlType (void) const { return (m_ControlType); }

	/* Get array elements */
  nifkfroteul_t* GetKeyRotEul (const int Index);
  nifkfrotlin_t* GetKeyRotLin (const int Index);
  nifkfrottcb_t* GetKeyRotTcb (const int Index);
  nifkfrotunk_t* GetKeyRotUnk (const int Index);
  nifkfposlin_t* GetKeyPosLin (const int Index);
  nifkfpostcb_t* GetKeyPosTcb (const int Index);
  nifkfposbez_t* GetKeyPosBez (const int Index);
  nifkfscalin_t* GetKeyScaLin (const int Index);
  nifkfscatcb_t* GetKeyScaTcb (const int Index);
  nifkfscabez_t* GetKeyScaBez (const int Index);

	/* Input/Output the section from the current position in the file */
  virtual bool Read  (CGenFile& FileHandle);
  virtual bool Write (CGenFile& FileHandle);

	/* Set class members */
  void SetRecordSize (const long Size) { m_RecordSize = Size; }
  void SetType       (const int Value) { m_Type       = Value; }

 };
/*===========================================================================
 *		End of Class CNifKeyFrameDataArray Definition
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifKeyFrameData Definition
 *
 * Description
 *
 *=========================================================================*/
class CNifKeyFrameData : public CNifBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CNifKeyFrameDataArray m_RotArray;
  CNifKeyFrameDataArray m_PosArray;
  CNifKeyFrameDataArray m_ScaArray;
  

  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CNifKeyFrameData();
  virtual ~CNifKeyFrameData() { Destroy(); }
  virtual void Destroy (void);

	/* Get class members */
  CNifKeyFrameDataArray* GetPositionData (void) { return (&m_PosArray); }
  CNifKeyFrameDataArray* GetRotationData (void) { return (&m_RotArray); }
  CNifKeyFrameDataArray* GetScalingData  (void) { return (&m_ScaArray); }

	/* Input/Output the section from the current position in the file */
  virtual bool Read  (CGenFile& FileHandle);
  virtual bool Write (CGenFile& FileHandle);

 };
/*===========================================================================
 *		End of Class CNifKeyFrameData Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File NifKeyFrameData.H
 *=========================================================================*/
