/*===========================================================================
 *
 * File:	Nifmaterialproperty.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "nifmaterialproperty.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifMaterialProperty.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMaterialProperty Constructor
 *
 *=========================================================================*/
CNifMaterialProperty::CNifMaterialProperty () {
  //DEFINE_FUNCTION("CNifMaterialProperty::CNifMaterialProperty()");
  m_TextureName = _T(""); 
 }
/*===========================================================================
 *		End of Class CNifMaterialProperty Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMaterialProperty Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifMaterialProperty::Destroy (void) {
  //DEFINE_FUNCTION("CNifMaterialProperty::Destroy()");
  m_TextureName = _T("");

	/* Call the base class method */
  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifMaterialProperty::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMaterialProperty Method - void AutoTint (void);
 *
 * Sets the ideal tints for the material.
 *
 *=========================================================================*/
void CNifMaterialProperty::AutoTint (void) {
  //DEFINE_FUNCTION("CNifMaterialProperty::AutoTint()");

  SetAmbientColor(NIF_MATERIAL_BEST_AMBIENT);
  SetDiffuseColor(NIF_MATERIAL_BEST_DIFFUSE);
  SetSpecularColor(NIF_MATERIAL_BEST_SPECULAR);
 }
/*===========================================================================
 *		End of Class Method CNifMaterialProperty::AutoTint()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMaterialProperty Method - long GetAmbientColor (void) const;
 *
 * Get the various colors in a COLORREF format.
 *
 *=========================================================================*/
long CNifMaterialProperty::GetAmbientColor (void) const {
  long Red   = (long) (m_Values[NIF_MATERIAL_AMBIENT_R]*255);
  long Green = (long) (m_Values[NIF_MATERIAL_AMBIENT_G]*255);
  long Blue  = (long) (m_Values[NIF_MATERIAL_AMBIENT_B]*255);

  return DL_RGB(Red, Green, Blue);
 }


long CNifMaterialProperty::GetDiffuseColor (void) const {
  long Red   = (long) (m_Values[NIF_MATERIAL_DIFFUSE_R]*255);
  long Green = (long) (m_Values[NIF_MATERIAL_DIFFUSE_G]*255);
  long Blue  = (long) (m_Values[NIF_MATERIAL_DIFFUSE_B]*255);

  return DL_RGB(Red, Green, Blue);
 }


long CNifMaterialProperty::GetSpecularColor (void) const {
  long Red   = (long) (m_Values[NIF_MATERIAL_SPECULAR_R]*255);
  long Green = (long) (m_Values[NIF_MATERIAL_SPECULAR_G]*255);
  long Blue  = (long) (m_Values[NIF_MATERIAL_SPECULAR_B]*255);

  return DL_RGB(Red, Green, Blue);
 }

long CNifMaterialProperty::GetIllumColor (void) const {
  long Red   = (long) (m_Values[NIF_MATERIAL_ILLUM_R]*255);
  long Green = (long) (m_Values[NIF_MATERIAL_ILLUM_G]*255);
  long Blue  = (long) (m_Values[NIF_MATERIAL_ILLUM_B]*255);

  return DL_RGB(Red, Green, Blue);
 }
/*===========================================================================
 *		End of Class Method CNifMaterialProperty::GetAmbientColor()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMaterialProperty Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifMaterialProperty::Read (CGenFile& FileHandle) {
  bool Result;
  
	/* Delete the current record contents */ 
  Destroy();

	/* Input the optional texture string name */
  Result = NifReadString(FileHandle, m_TextureName);
  if (!Result) return (false);
  
	/* Input first three numbers */
  Result = FileHandle.ReadLong(m_Unknown1);
  if (Result) Result = FileHandle.ReadLong(m_Unknown2);
  if (Result) Result = FileHandle.ReadShort(m_Unknown3);
  if (!Result) return (false);

	/* Input the material parameters */
  Result = FileHandle.Read((char *)&m_Values[0], sizeof(float)*NIF_MATERIAL_NUMVALUES);
  if (!Result) return (false);

	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifMaterialProperty::Read()
 *=========================================================================*/
 
  
/*===========================================================================
 *
 * Class CNifMaterialProperty Method - void SetAmbientColor (Color);
 *
 * Sets the colors from a COLORREF type value.
 *
 *=========================================================================*/
void CNifMaterialProperty::SetAmbientColor (const long Color) {
  m_Values[NIF_MATERIAL_AMBIENT_R] = DL_RED(Color)/255.0f;
  m_Values[NIF_MATERIAL_AMBIENT_G] = DL_GREEN(Color)/255.0f;
  m_Values[NIF_MATERIAL_AMBIENT_B] = DL_BLUE(Color)/255.0f;
 }


void CNifMaterialProperty::SetDiffuseColor (const long Color) {
  m_Values[NIF_MATERIAL_DIFFUSE_R] = DL_RED(Color)/255.0f;
  m_Values[NIF_MATERIAL_DIFFUSE_G] = DL_GREEN(Color)/255.0f;
  m_Values[NIF_MATERIAL_DIFFUSE_B] = DL_BLUE(Color)/255.0f;
 }


void CNifMaterialProperty::SetSpecularColor (const long Color) {
  m_Values[NIF_MATERIAL_SPECULAR_R] = DL_RED(Color)/255.0f;
  m_Values[NIF_MATERIAL_SPECULAR_G] = DL_GREEN(Color)/255.0f;
  m_Values[NIF_MATERIAL_SPECULAR_B] = DL_BLUE(Color)/255.0f;
 }
/*===========================================================================
 *		End of Class Method CNifMaterialProperty::SetAmbientColor()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMaterialProperty Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifMaterialProperty::Write (CGenFile& FileHandle) {
  bool Result;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

	/* Output the optional texture string name */
  Result = NifWriteString(FileHandle, m_TextureName);
  if (!Result) return (false);
  
	/* Output first three numbers */
  Result = FileHandle.WriteLong(m_Unknown1);
  if (Result) Result = FileHandle.WriteLong(m_Unknown2);
  if (Result) Result = FileHandle.WriteShort(m_Unknown3);
  if (!Result) return (false);

	/* Output the material parameters */
  Result = FileHandle.Write((char *)&m_Values[0], sizeof(float)*NIF_MATERIAL_NUMVALUES);
  if (!Result) return (false);

	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifMaterialProperty::Write()
 *=========================================================================*/
