/*===========================================================================
 *
 * File:	Nifmaterialproperty.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __NIFMATERIALPROPERTY_H
#define __NIFMATERIALPROPERTY_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "NifBase.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Number of values in the material data */
  #define NIF_MATERIAL_NUMVALUES 14
	
	/* Array indices */
  #define NIF_MATERIAL_AMBIENT_R  0
  #define NIF_MATERIAL_AMBIENT_G  1
  #define NIF_MATERIAL_AMBIENT_B  2
  #define NIF_MATERIAL_DIFFUSE_R  3
  #define NIF_MATERIAL_DIFFUSE_G  4
  #define NIF_MATERIAL_DIFFUSE_B  5
  #define NIF_MATERIAL_SPECULAR_R 6
  #define NIF_MATERIAL_SPECULAR_G 7
  #define NIF_MATERIAL_SPECULAR_B 8
  #define NIF_MATERIAL_ILLUM_R    9
  #define NIF_MATERIAL_ILLUM_G    10
  #define NIF_MATERIAL_ILLUM_B    11
  #define NIF_MATERIAL_GLOSSINESS 12
  #define NIF_MATERIAL_OPACITY    13

  	/* Ideal tint colors */
  #define NIF_MATERIAL_BEST_AMBIENT	RGB(128,128,128)
  #define NIF_MATERIAL_BEST_DIFFUSE	RGB(255,255,255)
  #define NIF_MATERIAL_BEST_SPECULAR	RGB(255,255,255)

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifMaterialProperty Definition
 *
 * Description
 *
 *=========================================================================*/
class CNifMaterialProperty : public CNifBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CString	m_TextureName;		/* Name of optional texture */

  long		m_Unknown1;		/* Unknown numbers */
  long		m_Unknown2;	
  short		m_Unknown3;

  float		m_Values[NIF_MATERIAL_NUMVALUES];	/* Material parameters */


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CNifMaterialProperty();
  virtual ~CNifMaterialProperty() { Destroy(); }
  virtual void Destroy (void);

	/* Sets the ideal tints for the material */
  void AutoTint (void);

  	/* Return the optional texture name */
  virtual const char*  GetRecordName  (void) const { return (m_TextureName); }

	/* Get color members */
  float GetAmbientRed    (void) const { return (m_Values[NIF_MATERIAL_AMBIENT_R]); }
  float GetAmbientGreen  (void) const { return (m_Values[NIF_MATERIAL_AMBIENT_G]); }
  float GetAmbientBlue   (void) const { return (m_Values[NIF_MATERIAL_AMBIENT_B]); }
  float GetDiffuseRed    (void) const { return (m_Values[NIF_MATERIAL_DIFFUSE_R]); }
  float GetDiffuseGreen  (void) const { return (m_Values[NIF_MATERIAL_DIFFUSE_G]); }
  float GetDiffuseBlue   (void) const { return (m_Values[NIF_MATERIAL_DIFFUSE_B]); }
  float GetSpecularRed   (void) const { return (m_Values[NIF_MATERIAL_SPECULAR_R]); }
  float GetSpecularGreen (void) const { return (m_Values[NIF_MATERIAL_SPECULAR_G]); }
  float GetSpecularBlue  (void) const { return (m_Values[NIF_MATERIAL_SPECULAR_B]); }
  float GetIllumRed      (void) const { return (m_Values[NIF_MATERIAL_ILLUM_R]); }
  float GetIllumGreen    (void) const { return (m_Values[NIF_MATERIAL_ILLUM_G]); }
  float GetIllumBlue     (void) const { return (m_Values[NIF_MATERIAL_ILLUM_B]); }
  float GetGlossiness    (void) const { return (m_Values[NIF_MATERIAL_GLOSSINESS]); }
  float GetOpacity       (void) const { return (m_Values[NIF_MATERIAL_OPACITY]); }
  long  GetAmbientColor  (void) const;
  long  GetDiffuseColor  (void) const;
  long  GetSpecularColor (void) const;
  long  GetIllumColor    (void) const;

	/* Input/Output the section from the current position in the file */
  virtual bool Read  (CGenFile& FileHandle);
  virtual bool Write (CGenFile& FileHandle);

	/* Set color members */
  void SetAmbientRed     (const float Value) { m_Values[NIF_MATERIAL_AMBIENT_R] = Value; }
  void SetAmbientGreen   (const float Value) { m_Values[NIF_MATERIAL_AMBIENT_G] = Value; }
  void SetAmbientBlue    (const float Value) { m_Values[NIF_MATERIAL_AMBIENT_B] = Value; }
  void SetDiffuseRed     (const float Value) { m_Values[NIF_MATERIAL_DIFFUSE_R] = Value; }
  void SetDiffuseGreen   (const float Value) { m_Values[NIF_MATERIAL_DIFFUSE_G] = Value; }
  void SetDiffuseBlue    (const float Value) { m_Values[NIF_MATERIAL_DIFFUSE_B] = Value; }
  void SetSpecularRed    (const float Value) { m_Values[NIF_MATERIAL_SPECULAR_R] = Value; }
  void SetSpecularGreen  (const float Value) { m_Values[NIF_MATERIAL_SPECULAR_G] = Value; }
  void SetSpecularBlue   (const float Value) { m_Values[NIF_MATERIAL_SPECULAR_B] = Value; }
  void SetAmbientColor   (const long Color);
  void SetDiffuseColor   (const long Color);
  void SetSpecularColor  (const long Color);

 };
/*===========================================================================
 *		End of Class CNifMaterialProperty Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Nifmaterialproperty.H
 *=========================================================================*/
