/*===========================================================================
 *
 * File:	NifMorphData.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Tuesday, December 24, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "NifMorphData.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifMorphData.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphDataRecord Constructor
 *
 *=========================================================================*/
CNifMorphDataRecord::CNifMorphDataRecord () {
  //DEFINE_FUNCTION("CNifMorphDataRecord::CNifMorphDataRecord()");
  m_NumPoints4 = 0;
  m_pFloats = NULL;
 }
/*===========================================================================
 *		End of Class CNifMorphDataRecord Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphDataRecord Method - void Destroy (void);
 *
 * Description
 *
 *=========================================================================*/
void CNifMorphDataRecord::Destroy (void) {
  DEFINE_FUNCTION("CNifMorphDataRecord::Destroy()");

  DestroyArrayPointer(m_pFloats);
  m_NumPoints4 = 0;
 }
/*===========================================================================
 *		End of Class Method CNifMorphDataRecord::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphDataRecord Method - bool Read (CGenFile& FileHandle);
 *
 * Description
 *
 *=========================================================================*/
bool CNifMorphDataRecord::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifMorphDataRecord::Read()");
  bool Result;
  long NumFloats;

  Destroy();

  Result = FileHandle.ReadLong(m_NumPoints4);
  if (Result) Result = FileHandle.ReadLong(m_Unknown1);
  if (!Result) return (false);

  NumFloats = GetNumFloats();

  if (NumFloats > 0) {
    CreateArrayPointer(m_pFloats, float, NumFloats);
    Result = FileHandle.Read((char*)m_pFloats, sizeof(float)*NumFloats);
    if (!Result) return (false);
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifMorphDataRecord::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphDataRecord Method - bool Write (CGenFile& FileHandle);
 *
 * Description
 *
 *=========================================================================*/
bool CNifMorphDataRecord::Write (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifMorphDataRecord::Write()");
  bool Result;
  long NumFloats;

  Result = FileHandle.WriteLong(m_NumPoints4);
  if (Result) Result = FileHandle.WriteLong(m_Unknown1);
  if (!Result) return (false);

  NumFloats = GetNumFloats();

  if (NumFloats > 0) {
    Result = FileHandle.Write((char*)m_pFloats, sizeof(float)*NumFloats);
    if (!Result) return (false);
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifMorphDataRecord::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphData Constructor
 *
 *=========================================================================*/
CNifMorphData::CNifMorphData () {
  //DEFINE_FUNCTION("CNifMorphData::CNifMorphData()");
  m_pRecords = NULL;
  m_NumRecords = 0;
 }
/*===========================================================================
 *		End of Class CNifMorphData Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphData Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifMorphData::Destroy (void) {
  DEFINE_FUNCTION("CNifMorphData::Destroy()");
  
  m_NumRecords = 0;
  DestroyArrayPointer(m_pRecords);

  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifMorphData::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphData Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifMorphData::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifMorphData::Read()");
  bool Result;
  int  Index;

  Destroy();

  Result = FileHandle.ReadLong(m_NumRecords);
  if (Result) Result = FileHandle.ReadLong(m_NumPoints);
  if (Result) Result = FileHandle.ReadChar(m_Unknown1);
  if (!Result) return (false);
  
  if (m_NumRecords > 0) {
    CreateArrayPointer(m_pRecords, CNifMorphDataRecord, m_NumRecords);

    for (Index = 0; Index < m_NumRecords; Index++) {
      m_pRecords[Index].SetNumPoints3(m_NumPoints);
      Result = m_pRecords[Index].Read(FileHandle);
      if (!Result) return (false);
     }
   }

  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifMorphData::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifMorphData Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifMorphData::Write (CGenFile& FileHandle) {
  //DEFINE_FUNCTION("CNifMorphData::Write()");
  bool Result;
  int  Index;

	/* Call the base class record method */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

  Result = FileHandle.WriteLong(m_NumRecords);
  if (Result) Result = FileHandle.WriteLong(m_NumPoints);
  if (Result) Result = FileHandle.WriteChar(m_Unknown1);
  if (!Result) return (false);
  
  if (m_NumRecords > 0) {

    for (Index = 0; Index < m_NumRecords; Index++) {
      Result = m_pRecords[Index].Write(FileHandle);
      if (!Result) return (false);
     }
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifMorphData::Write()
 *=========================================================================*/