/*===========================================================================
 *
 * File:	Nifnode.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Defines the CNifNode file.
 *
 *=========================================================================*/
#ifndef __NIFNODE_H
#define __NIFNODE_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "NifBase.h"
  #include "NifLongArray.h"
  #include "NifFloatArray.h" 
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Number of axis coordinates contained in the node data */
  #define NIF_NODE_NUMPOINTS 16
  #define NIF_NODE_AXISINDEX_POS 0

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifNode Definition
 *
 * Description
 *
 *=========================================================================*/
class CNifNode : public CNifBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CString	m_Name;				/* Node name */

  long		m_Unknown1;			/* Unknown data (0xFFFFFFFF)? */
  long		m_Unknown2;			/* Unknown data (0xFFFFFFFF)? */
  short		m_Unknown3;			/* Count of something? */

  float		m_Points[NIF_NODE_NUMPOINTS];	/* Set of coordinates */

  CNifLongArray  m_Array1;
  CNifFloatArray m_Array2;
  CNifLongArray  m_ChildArray;
  CNifLongArray  m_Array4;
  

  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CNifNode();
  virtual ~CNifNode() { Destroy(); }
  virtual void Destroy (void);

  	/* Return the optional record name */
  virtual const char*	GetRecordName	(void) const { return (m_Name); }

	/* Set class members */
  float		 GetFloat         (const int Index) const { return ((Index >= 0 && Index < NIF_NODE_NUMPOINTS) ? m_Points[Index] : 0); }
  float*	 GetFloatData     (void)		  { return (&m_Points[0]); }
  CNifLongArray* GetChildArray    (void)		  { return (&m_ChildArray); }
  long		 GetKeyFrameIndex (void)		  { return (m_Unknown2); }	

	/* Mod the node's position */
  void ModPos	   (const int Flags, const float Delta);
  void ModPosWorld (const int Flags, const float Delta);

	/* Input/Output the section from the current position in the file */
  virtual bool Read  (CGenFile& FileHandle);
  virtual bool Write (CGenFile& FileHandle);

	/* Set class members */
  void SetFloat (const int Index, const float Value) { if (Index >= 0 && Index < NIF_NODE_NUMPOINTS) m_Points[Index] = Value; }

 };
/*===========================================================================
 *		End of Class CNifNode Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Nifnode.H
 *=========================================================================*/
