/*===========================================================================
 *
 * File:	NifSkinData.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Tuesday, December 24, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "NifSkinData.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifSkinData.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinDataRecord Constructor
 *
 *=========================================================================*/
CNifSkinDataRecord::CNifSkinDataRecord () {
  //DEFINE_FUNCTION("CNifSkinDataRecord::CNifSkinDataRecord()");
  m_ArraySize = 0;
  m_pArray = NULL;
 }
/*===========================================================================
 *		End of Class CNifSkinDataRecord Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinDataRecord Method - void Destroy (void);
 *
 * Description
 *
 *=========================================================================*/
void CNifSkinDataRecord::Destroy (void) {
  DEFINE_FUNCTION("CNifSkinDataRecord::Destroy()");

  DestroyArrayPointer(m_pArray);
  m_ArraySize = 0;
 }
/*===========================================================================
 *		End of Class Method CNifSkinDataRecord::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinDataRecord Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifSkinDataRecord::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifSkinDataRecord::Read()");
  bool Result;

	/* Clear the current contents */
  Destroy();
  //SystemLog.Printf("\t\tSkinDataRecord1 = 0x%08lX", FileHandle.Tell());
  Result = FileHandle.Read((char *)&m_Floats[0], sizeof(float)*NIF_SKINDATARECORD_NUMFLOATS);
  //SystemLog.Printf("\t\tSkinDataRecord2 = 0x%08lX", FileHandle.Tell());
  if (Result) Result = FileHandle.ReadShort(m_ArraySize);
  if (!Result) return (false);

  if (m_ArraySize < 0) {
    ErrorHandler.AddError(ERR_CUSTOM, "Invalid skin record size %d (0x%08lX)", m_ArraySize, FileHandle.Tell()-2);
    return (false);
   }

	/* Allocate and input the array */
  if (m_ArraySize > 0) {
    CreateArrayPointer(m_pArray, nifskindatagroup_t, m_ArraySize);
    Result = FileHandle.Read((char *)m_pArray, sizeof(nifskindatagroup_t)*m_ArraySize);
    if (!Result) return (false);
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifSkinDataRecord::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinDataRecord Method - bool Write (FileHandle);
 *
 *=========================================================================*/
bool CNifSkinDataRecord::Write (CGenFile& FileHandle) {
  //DEFINE_FUNCTION("CNifSkinDataRecord::Write()");
  bool Result;

  Result = FileHandle.Write((char *)&m_Floats[0], sizeof(float)*NIF_SKINDATARECORD_NUMFLOATS);
  if (Result) Result = FileHandle.WriteShort(m_ArraySize);
  if (!Result) return (false);

	/* Output the array */
  if (m_ArraySize > 0) {
    Result = FileHandle.Write((char *)m_pArray, sizeof(nifskindatagroup_t)*m_ArraySize);
    if (!Result) return (false);
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifSkinDataRecord::Write()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinData Constructor
 *
 *=========================================================================*/
CNifSkinData::CNifSkinData () {
  //DEFINE_FUNCTION("CNifSkinData::CNifSkinData()");
  m_NumRecords = 0;
  m_pRecords = NULL; 
 }
/*===========================================================================
 *		End of Class CNifSkinData Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinData Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifSkinData::Destroy (void) {
 DEFINE_FUNCTION("CNifSkinData::Destroy()");
  DestroyArrayPointer(m_pRecords);
  m_NumRecords = 0;

  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifSkinData::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinData Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifSkinData::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifSkinData::Read()");
  bool Result;
  int  Index;

  Destroy();

  Result = FileHandle.Read((char *)&m_Floats[0], NIF_SKINDATA_NUMFLOATS*sizeof(float));
  if (Result) Result = FileHandle.ReadLong(m_NumRecords);
  if (Result) Result = FileHandle.ReadLong(m_Unknown1);
  if (!Result) return (false);

  if (m_NumRecords < 0) {
    ErrorHandler.AddError(ERR_CUSTOM, "Invalid number of skin records %d (0x%08lX)", m_NumRecords, FileHandle.Tell()-8);
    return (false);
   }

  if (m_NumRecords > 0) {
    CreateArrayPointer(m_pRecords, CNifSkinDataRecord, m_NumRecords);

    for (Index = 0; Index < m_NumRecords; Index++) {
      Result = m_pRecords[Index].Read(FileHandle);
      if (!Result) return (false);
     }
   }

  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifSkinData::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifSkinData Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifSkinData::Write (CGenFile& FileHandle) {
  //DEFINE_FUNCTION("CNifSkinData::Write()");
  bool Result;
  int  Index;

	/* Call the base class record method */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

  Result = FileHandle.Write((char *)&m_Floats[0], NIF_SKINDATA_NUMFLOATS*sizeof(float));
  if (Result) Result = FileHandle.WriteLong(m_NumRecords);
  if (Result) Result = FileHandle.WriteLong(m_Unknown1);
  if (!Result) return (false);

  if (m_NumRecords > 0) {
    for (Index = 0; Index < m_NumRecords; Index++) {
      Result = m_pRecords[Index].Write(FileHandle);
      if (!Result) return (false);
     }
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifSkinData::Write()
 *=========================================================================*/