/*===========================================================================
 *
 * File:	Niftrishape.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "niftrishape.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifTriShape.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShape Constructor
 *
 *=========================================================================*/
CNifTriShape::CNifTriShape () {
  //DEFINE_FUNCTION("CNifTriShape::CNifTriShape()");

 }
/*===========================================================================
 *		End of Class CNifTriShape Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShape Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifTriShape::Destroy (void) {
  //DEFINE_FUNCTION("CNifTriShape::Destroy()");

  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifTriShape::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShape Method - void ModPos (Flags, Delta);
 *
 *=========================================================================*/
void CNifTriShape::ModPos (const int Flags, const float Delta) {
  if (Flags & NIF_MODX) m_Coordinates[0] += Delta;
  if (Flags & NIF_MODY) m_Coordinates[1] += Delta;
  if (Flags & NIF_MODZ) m_Coordinates[2] += Delta;
 }
/*===========================================================================
 *		End of Class Method CNifTriShape::ModPos()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShape Method - void ModScale (Delta);
 *
 *=========================================================================*/
void CNifTriShape::ModScale (const float Delta) {
  m_Coordinates[12] *= Delta;
 }
/*===========================================================================
 *		End of Class Method CNifTriShape::ModScale()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShape Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifTriShape::Read (CGenFile& FileHandle) {
  bool Result;
  
	/* Delete the current record contents */ 
  Destroy();

	/* Output the node name */
  Result = NifReadString(FileHandle, m_ShapeName);
  if (!Result) return (false);

  	/* Input first three numbers */
  Result = FileHandle.ReadLong(m_Unknown1);
  if (Result) Result = FileHandle.ReadLong(m_Unknown2);
  if (Result) Result = FileHandle.ReadShort(m_Unknown3);
  if (!Result) return (false);

	/* Input coordinates */
  Result = FileHandle.Read((char *)&m_Coordinates[0], sizeof(float)*NIF_TRISHAPE_NUMCOORS);
  if (!Result) return (false);

  Result = m_Array.Read(FileHandle);
  if (Result) Result = FileHandle.ReadLong(m_Unknown4);
  if (Result) Result = FileHandle.ReadLong(m_TriShapeDataIndex);
  if (Result) Result = FileHandle.ReadLong(m_Unknown6);
  if (!Result) return (false);
  
	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifTriShape::Read()
 *=========================================================================*/
  

/*===========================================================================
 *
 * Class CNifTriShape Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifTriShape::Write (CGenFile& FileHandle) {
  bool Result;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

	/* Output the node name */
  Result = NifWriteString(FileHandle, m_ShapeName);
  if (!Result) return (false);

  	/* Output first three numbers */
  Result = FileHandle.WriteLong(m_Unknown1);
  if (Result) Result = FileHandle.WriteLong(m_Unknown2);
  if (Result) Result = FileHandle.WriteShort(m_Unknown3);
  if (!Result) return (false);

	/* Output coordinates */
  Result = FileHandle.Write((char *)&m_Coordinates[0], sizeof(float)*NIF_TRISHAPE_NUMCOORS);
  if (!Result) return (false);

  	/* Output indices */
  Result = m_Array.Write(FileHandle);
  if (Result) Result = FileHandle.WriteLong(m_Unknown4);
  if (Result) Result = FileHandle.WriteLong(m_TriShapeDataIndex);
  if (Result) Result = FileHandle.WriteLong(m_Unknown6);
  if (!Result) return (false);

	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifTriShape::Write()
 *=========================================================================*/



