/*===========================================================================
 *
 * File:	Niftrishapedata.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 * 25 October 2003
 *	- Fixed input/output with more than 3 sub-records section at the end.
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "niftrishapedata.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifTriShapedata.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Constructor
 *
 *=========================================================================*/
CNifTriShapeData::CNifTriShapeData () {
  //DEFINE_FUNCTION("CNifTriShapeData::CNifTriShapeData()");
  m_Unknown1 = 0;
  m_Unknown2 = 0;

  m_pPoints         = NULL;
  m_pNormals        = NULL;
  m_pFaceIndices    = NULL;
  m_pVertexColors   = NULL;
  m_pUnknownPoints2 = NULL;

  m_NumPoints  = 0;
  m_NumFaces   = 0;
  m_NumFaceIndices = 0;

  m_NumRecords = 0;
  m_pRecords   = NULL;

  m_CalcMinPosition = false;
  m_CalcMaxPosition = false;
 }
/*===========================================================================
 *		End of Class CNifTriShapeData Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifTriShapeData::Destroy (void) {
  DEFINE_FUNCTION("CNifTriShapeData::Destroy()");

  DestroyArrayPointer(m_pPoints);
  DestroyArrayPointer(m_pNormals);
  DestroyArrayPointer(m_pFaceIndices);
  DestroyArrayPointer(m_pVertexColors);
  DestroyArrayPointer(m_pUnknownPoints2);
  DestroyArrayPointer(m_pRecords);
  m_NumRecords = 0;
  m_NumPoints  = 0;
  m_NumFaces   = 0;
  m_NumFaceIndices = 0;
  m_CalcMinPosition = false;
  m_CalcMaxPosition = false;

	/* Call the base class method */
  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void AllocatePoints (void);
 *
 * Protected class method to allocate the point/normals array.
 *
 *=========================================================================*/
void CNifTriShapeData::AllocatePoints (void) {
  DEFINE_FUNCTION("CNifTriShapeData::AllocatePoints()");

	/* Delete the current arrays, if any */
  DestroyArrayPointer(m_pPoints);
  DestroyArrayPointer(m_pNormals);

	/* Test for valid point counter */
  ASSERT(m_NumPoints >= 0);
  if (m_NumPoints == 0) return;

	/* Allocate the new arrays */
  CreateArrayPointer(m_pPoints,  nifpos_t, m_NumPoints);
  CreateArrayPointer(m_pNormals, nifpos_t, m_NumPoints);
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::AllocatePoints()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void AllocateFaceIndices (void);
 *
 * Protected class method to allocate the face index array.
 *
 *=========================================================================*/
void CNifTriShapeData::AllocateFaceIndices (void) {
  DEFINE_FUNCTION("CNifTriShapeData::AllocateFaceIndices()");

	/* Delete the current arrays, if any */
  DestroyArrayPointer(m_pFaceIndices);

	/* Test for valid point counter */
  ASSERT(m_NumFaceIndices >= 0);
  if (m_NumFaceIndices == 0) return;

	/* Allocate the new arrays */
  CreateArrayPointer(m_pFaceIndices, short, m_NumFaceIndices);
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::AllocateFaceIndices()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void AllocateVertexColors (void);
 *
 * Description
 *
 *=========================================================================*/
void CNifTriShapeData::AllocateVertexColors (void) {
  DEFINE_FUNCTION("CNifTriShapeData::AllocateVertexColors()");

	/* Delete the current arrays, if any */
  DestroyArrayPointer(m_pVertexColors);

	/* Test for valid point counter */
  //ASSERT(m_NumPoints >= 0);
  if (m_NumPoints == 0) return;

	/* Allocate the new arrays */
  CreateArrayPointer(m_pVertexColors, nifcolor4_t, m_NumPoints);
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::AllocateVertexColors()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void AllocateUnknownPoints2 (void);
 *
 * Description
 *
 *=========================================================================*/
void CNifTriShapeData::AllocateUnknownPoints2 (void) {
  DEFINE_FUNCTION("CNifTriShapeData::AllocateUnknownPoints2()");

	/* Delete the current arrays, if any */
  DestroyArrayPointer(m_pUnknownPoints2);

	/* Test for valid point counter */
  //ASSERT(m_NumPoints >= 0);
  if (m_NumPoints == 0) return;

	/* Allocate the new arrays */
  if (m_Unknown4[2] == 2) {
    CreateArrayPointer(m_pUnknownPoints2, float, m_NumPoints*4);
   }
  else {
    CreateArrayPointer(m_pUnknownPoints2, float, m_NumPoints*2);
   }
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::AllocateUnknownPoints2()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void GetMinPos (Position);
 *
 * Computes and returns the minimum point position.
 *
 *=========================================================================*/
void CNifTriShapeData::GetMinPos (nifpos_t& Position) {
  int Index;

	/* If min position already computed, just return it */
  if (m_CalcMinPosition) {
    Position = m_MinPosition;
    return;
   }

	/* Check for special case of no points */
  if (m_NumPoints == 0) {
    Position.X = 0;
    Position.Y = 0;
    Position.Z = 0;
    return;
   }

  Position = m_pPoints[0];

	/* Find the minimum XYZ position */
  for (Index = 1; Index < m_NumPoints; Index++) {
    if (m_pPoints[Index].X < Position.X) Position.X = m_pPoints[Index].X;
    if (m_pPoints[Index].Y < Position.Y) Position.Y = m_pPoints[Index].Y;
    if (m_pPoints[Index].Z < Position.Z) Position.Z = m_pPoints[Index].Z;
   }

  m_MinPosition = Position;
  m_CalcMinPosition = true;
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::GetMinPos()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void GetMaxPos (Position);
 *
 * Computes and returns the maximum point position.
 *
 *=========================================================================*/
void CNifTriShapeData::GetMaxPos (nifpos_t& Position) {
  int Index;

	/* If max position already computed, just return it */
  if (m_CalcMaxPosition) {
    Position = m_MaxPosition;
    return;
   }

	/* Check for special case of no points */
  if (m_NumPoints == 0) {
    Position.X = 0;
    Position.Y = 0;
    Position.Z = 0;
    return;
   }

  Position = m_pPoints[0];

	/* Find the minimum XYZ position */
  for (Index = 1; Index < m_NumPoints; Index++) {
    if (m_pPoints[Index].X > Position.X) Position.X = m_pPoints[Index].X;
    if (m_pPoints[Index].Y > Position.Y) Position.Y = m_pPoints[Index].Y;
    if (m_pPoints[Index].Z > Position.Z) Position.Z = m_pPoints[Index].Z;
   }

  m_MaxPosition = Position;
  m_CalcMaxPosition = true;
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::GetMaxPos()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - void ModPos (Flags, Delta);
 *
 * Description
 *
 *=========================================================================*/
void CNifTriShapeData::ModPos (const int Flags, const float Delta) {
  int Index;

  for (Index = 0; Index < m_NumPoints; Index++) {
    if (Flags & NIF_MODX) m_pPoints[Index].X += Delta;
    if (Flags & NIF_MODY) m_pPoints[Index].Y += Delta;
    if (Flags & NIF_MODZ) m_pPoints[Index].Z += Delta;
   }

  m_CalcMaxPosition = false;
  m_CalcMinPosition = false;
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::ModPos()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifTriShapeData Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifTriShapeData::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifTriShapeData::Read()");
  bool Result;
  int  Index;
  
	/* Delete the current record contents */ 
  Destroy();

	/* Input the first two unknowns */
  Result = FileHandle.ReadShort(m_NumPoints);
  if (Result) Result = FileHandle.ReadFloat(m_Unknown1);
  //SystemLog.Printf("\t\tNumPoints = %d", (int)m_NumPoints);
  //SystemLog.Printf("\t\tUnknown1 = %f", m_Unknown1);
  
	/* Input the points/normals array */
  AllocatePoints();
  Result = FileHandle.Read((char *)&m_pPoints[0], sizeof(nifpos_t)*m_NumPoints);
  if (Result) Result = FileHandle.ReadFloat(m_Unknown2);
  if (!Result) return (false);
  //SystemLog.Printf("\t\tUnknown2 = %f", m_Unknown2);

  if (m_Unknown2 != 0) {
    Result = FileHandle.Read((char *)&m_pNormals[0], sizeof(nifpos_t)*m_NumPoints);
    //SystemLog.Printf ("\tAfter Points = 0x%08lX", FileHandle.Tell());
    if (!Result) return (false);
   }
  
	/* Input more unknowns */				/* 4 */
  Result = FileHandle.Read((char *)&m_Unknown3[0], sizeof(float)*NIF_TRISHAPE_NUMUNKNOWN3);
  if (Result) Result = FileHandle.ReadShort(m_Unknown4[0]);
  if (Result) Result = FileHandle.ReadShort(m_Unknown4[1]);
  //SystemLog.Printf("\t\tUnknown4 = %d %d", (int)m_Unknown4[0], (int)m_Unknown4[1]);
  //SystemLog.Printf ("\tAfter Unknown4(1) = 0x%08lX", FileHandle.Tell());
  if (!Result) return (false);

	/* Input the optional unknown points array if required */
  if (m_Unknown4[1] != 0) {
    //OutputLog.Printf ("U1 = 0x%04X", (int) m_Unknown4[1]);
    AllocateVertexColors();
    Result = FileHandle.Read((char *)&m_pVertexColors[0], sizeof(nifcolor4_t)*m_NumPoints);
    if (!Result) return (false);
    //SystemLog.Printf("\t\tHas Unknown Points1");
   }

  Result = FileHandle.ReadShort(m_Unknown4[2]);
  if (Result) Result = FileHandle.ReadShort(m_Unknown4[3]);
  if (Result) Result = FileHandle.ReadShort(m_Unknown4[4]);
  //SystemLog.Printf("\t\tUnknown4 = %d %d %d", (int)m_Unknown4[2], (int)m_Unknown4[3], (int)m_Unknown4[4]);
  //SystemLog.Printf ("\tAfter Unknown4(4) = 0x%08lX", FileHandle.Tell());
  if (!Result) return (false);

	/* Input the optional UV array */
  if (m_Unknown4[4] != 0) {
    AllocateUnknownPoints2();
    //OutputLog.Printf ("\t\tU4 = 0x%04X", (int) m_Unknown4[4]);

    if (m_Unknown4[2] == 2)
      Result = FileHandle.Read((char *)&m_pUnknownPoints2[0], sizeof(float)*m_NumPoints*4);
    else
      Result = FileHandle.Read((char *)&m_pUnknownPoints2[0], sizeof(float)*m_NumPoints*2);

    if (!Result) return (false);
    //SystemLog.Printf("\t\tHas Unknown Points3");
   }
  
	/* Input the face index array */
  Result = FileHandle.ReadShort(m_NumFaces);
  if (Result) Result = FileHandle.ReadLong(m_NumFaceIndices);
  //SystemLog.Printf("\t\tNumFaces = %d", (int)m_NumFaces);
  //SystemLog.Printf("\t\tNumFaceIndices = %d", (int)m_NumFaceIndices);
  //SystemLog.Printf ("\tAfter NumFaces = 0x%08lX", FileHandle.Tell());
  if (!Result) return (false);

  if (m_NumFaceIndices < 0 || m_NumFaces < 0) {
    ErrorHandler.AddError(ERR_CUSTOM, "Invalid face size (0x%08lX)!", FileHandle.Tell()-6);
    return (false);
   }

  AllocateFaceIndices();
  Result = FileHandle.Read((char *)&m_pFaceIndices[0], sizeof(short)*(m_NumFaceIndices));
  //SystemLog.Printf ("\tAfter FaceIndices = 0x%08lX", FileHandle.Tell());
  if (Result) Result = FileHandle.ReadShort(m_NumRecords);
  if (!Result) return (false);

  if (m_NumRecords > 0) {
    CreateArrayPointer(m_pRecords, CNifShortArray, m_NumRecords);

    for (Index = 0; Index < m_NumRecords; Index++) {
      //SystemLog.Printf ("\tBefore Record #%d = 0x%08lX", Index, FileHandle.Tell());
      Result = m_pRecords[Index].Read(FileHandle);
      if (!Result) return (false);
      //SystemLog.Printf ("\tAfter Record #%d = 0x%08lX", Index, FileHandle.Tell());
     }
   }

	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::Read()
 *=========================================================================*/
  

/*===========================================================================
 *
 * Class CNifTriShapeData Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifTriShapeData::Write (CGenFile& FileHandle) {
  bool Result;
  int  Index;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

	/* Output the first two unknowns */
  Result = FileHandle.WriteShort(m_NumPoints);
  if (Result) Result = FileHandle.WriteFloat(m_Unknown1);
  
	/* Output the points/normals array */
  if (m_NumPoints != 0) {
    Result = FileHandle.Write((char *)&m_pPoints[0], sizeof(nifpos_t)*m_NumPoints);
    if (Result) Result = FileHandle.WriteFloat(m_Unknown2);
    if (!Result) return (false);
    
    if (m_Unknown2 != 0) {
      Result = FileHandle.Write((char *)&m_pNormals[0], sizeof(nifpos_t)*m_NumPoints);
      if (!Result) return (false);
     }
   }
  else {
    Result = FileHandle.WriteFloat(m_Unknown2);
    if (!Result) return (false);
   }
  
	/* Output more unknowns */
  Result = FileHandle.Write((char *)&m_Unknown3[0], sizeof(float)*NIF_TRISHAPE_NUMUNKNOWN3);
  if (Result) Result = FileHandle.WriteShort(m_Unknown4[0]);
  if (Result) Result = FileHandle.WriteShort(m_Unknown4[1]);
  if (!Result) return (false);

	/* Output the unknown points array if required */
  if (m_Unknown4[1] != 0 && m_NumPoints > 0) {
    Result = FileHandle.Write((char *)&m_pVertexColors[0], sizeof(nifpos_t)*m_NumPoints);
    if (!Result) return (false);
   }

  Result = FileHandle.WriteShort(m_Unknown4[2]);
  if (Result) Result = FileHandle.WriteShort(m_Unknown4[3]);
  if (Result) Result = FileHandle.WriteShort(m_Unknown4[4]);
  if (!Result) return (false);
  
  if (m_Unknown4[4] != 0 && m_NumPoints > 0) {

    if (m_Unknown4[2] == 2)
      Result = FileHandle.Write((char *)&m_pUnknownPoints2[0], sizeof(float)*m_NumPoints*4);
    else
      Result = FileHandle.Write((char *)&m_pUnknownPoints2[0], sizeof(float)*m_NumPoints*2);

    if (!Result) return (false);
   }

	/* Output the face index array */
  Result = FileHandle.WriteShort(m_NumFaces);
  if (Result) Result = FileHandle.WriteLong(m_NumFaceIndices);

  if (m_NumFaces != 0) {
    if (Result) Result = FileHandle.Write((char *)&m_pFaceIndices[0], sizeof(short)*(m_NumFaceIndices));
   }

  if (Result) Result = FileHandle.WriteShort(m_NumRecords);
  if (!Result) return (false);

  for (Index = 0; Index < m_NumRecords; Index++) {
    Result = m_pRecords[Index].Write(FileHandle);
    if (!Result) return (false);
   }

	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifTriShapeData::Write()
 *=========================================================================*/




