/*===========================================================================
 *
 * File:	Niffile.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Defines the CNifFile class for handling the basic I/O of a NIF file.
 *
 *=========================================================================*/
#ifndef __NIFFILE_H
#define __NIFFILE_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "NifNode.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Return value from the find record methods */
  #define NIF_FINDRECORD_BADINDEX (-1)

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

	/* The NIF file header structure */
  typedef struct {
    char HeaderText[NIF_HEADER_NAMESIZE];	
    long Version;
    long NumRecords;
   } nifheader_t;

#pragma pack(pop)
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Record Names
 *
 *=========================================================================*/

  #define NIF_RECORD_NODE		"NiNode"
  #define NIF_RECORD_TRISHAPE		"NiTriShape"
  #define NIF_RECORD_MATERIALPROPERTY	"NiMaterialProperty"
  #define NIF_RECORD_TEXTURINGPROPERTY  "NiTexturingProperty"
  #define NIF_RECORD_ALPHAPROPERTY      "NiAlphaProperty"
  #define NIF_RECORD_SOURCETEXTURE	"NiSourceTexture"
  #define NIF_RECORD_SPECULARPROPERTY	"NiSpecularProperty"
  #define NIF_RECORD_ROOTCOLLISIONNODE	"RootCollisionNode"
  #define NIF_RECORD_KEYFRAMECONTROLLER	"NiKeyframeController"
  #define NIF_RECORD_KEYFRAMEDATA	"NiKeyframeData"

/*===========================================================================
 *		End of Record Names
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifFile Definition
 *
 * Description
 *
 *=========================================================================*/
class CNifFile : public CGenFile {

  /*---------- Begin Private Class Members ----------------------*/
private:
  nifheader_t	m_Header;		/* File header */
  bool		m_ReadHeader;		/* Flags to indicate portion of file read */

  CPtrList	m_Records;		/* List of NIF records in the file */


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Create the appropriate NIF record based on the input name */
  CNifBaseRecord* CreateRecord (const CString& RecordName, const long& Length);

	/* Helper input/output methods */
  bool ReadHeader  (void);
  bool WriteHeader (void);
  bool Read        (void);
  bool Write       (void);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CNifFile();
  virtual ~CNifFile() { Destroy(); }
  virtual void Destroy        (void);
  virtual void DeleteContents (void);

	/* Find records in the NIF file */
  CNifBaseRecord* FindFirstRecord (POSITION& Pos, const TCHAR* pName);
  CNifBaseRecord* FindNextRecord  (POSITION& Pos, const TCHAR* pName);

	/* Get records in the NIF file */
  CNifBaseRecord* GetFirstRecord (POSITION& Pos);
  CNifBaseRecord* GetNextRecord  (POSITION& Pos);
  CNifBaseRecord* GetRecord      (const int Index);
  int		  GetNumRecords  (void) const { return (m_Records.GetCount()); }

	/* Modify various record position data */
  void ModNodePos	  (const int Flags, const float Delta);
  void ModNodePosWorld	  (const int Flags, const float Delta);
  void ModTriShapePos	  (const int Flags, const float Delta);
  void ModTriShapeDataPos (const int Flags, const float Delta);
  void ModTriShapeScale   (const float Delta);
	
	/* Input/output methods */
  bool Read  (const char* pFilename);
  bool Write (const char* pFilename);

 };
/*===========================================================================
 *		End of Class CNifFile Definition
 *=========================================================================*/



#endif
/*===========================================================================
 *		End of File Niffile.H
 *=========================================================================*/

