/*===========================================================================
 *
 * File:	Mwesm_Alch.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordAlch class.
 *
 *=========================================================================*/
#ifndef __MWESM_ALCH_H
#define __MWESM_ALCH_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/


/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

	/* Alchemy data structure */
  typedef struct {
    float Weight;
    long  Value;
    long  AutoCalc;
   } mwesmalchdata_t;

	/* Alchemy ehchantment data structure */
  typedef struct {
    short EffectID;
    short SkillID;
    short AttributeID;
    short Unknown1;
    short Unknown2;
    short Unknown3;
    long  Duration;
    long  Magnitude;
    long  Unknown4;
   } mwesmalchenchant_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordAlch Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = Item ID, required
	MODL = Model Name, required
	FNAM = Item Name, required
	ALDT = Alchemy Data (12 bytes), required
		float  Weight
		long   Value
		long   AutoCalc
	ENAM = Enchantment (24 bytes) 1 to 8 per record
		short EffectID
		short SkillID		for skill related effects, -1/0 otherwise
		short AttributeID	for attribute related effects, -1/0 otherwise
		short Unknown1
		short Unknown2
		short Unknown3
		long  Duration
		long  Magnitude
		long  Unknown4
	ITEX = Inventory Icon
	SCRI = Script Name

 *
 *=========================================================================*/
class CMwEsmRecordAlch : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString		m_ID;
  CMwEsmString		m_Model;
  CMwEsmString		m_Name;
  CMwEsmString		m_Icon;
  CMwEsmString		m_Enchantment;
  CMwEsmString		m_Script;

  mwesmalchdata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordAlch();
  virtual ~CMwEsmRecordAlch() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */
  const char*  GetTypeString    (void) const { return ("Alchemy"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordAlch Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

