/*===========================================================================
 *
 * File:	Mwesm_Appa.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordAppa class.
 *
 *=========================================================================*/
#ifndef __MWESM_APPA_H
#define __MWESM_APPA_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/


/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

  typedef enum {
	MWESM_APPATYPE_MORTAR	  = 0,
	MWESM_APPATYPE_ALBEMIC	  = 1,
	MWESM_APPATYPE_CALCINATOR = 2,
	MWESM_APPATYPE_RETORT	  = 3 
    } mwesmappatype_t;

	/* Repair data structure */
  typedef struct {
    long  Type;
    float Quality;
    float Weight;
    long  Value;
   } mwesmappadata_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordAppa Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = Item ID, required
	MODL = Model Name, required
	FNAM = Item Name, required
	AADT = Alchemy Data (16 bytes), required
	ITEX = Inventory Icon
	SCRI = Script Name
 *
 *=========================================================================*/
class CMwEsmRecordAppa : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString		m_ID;
  CMwEsmString		m_Model;
  CMwEsmString		m_Name;
  CMwEsmString		m_Icon;
  CMwEsmString		m_Enchantment;
  CMwEsmString		m_Script;

  mwesmappadata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordAppa();
  virtual ~CMwEsmRecordAppa() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */
  const char*  GetTypeString    (void) const { return ("Apparatus"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordAppa Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

