/*===========================================================================
 *
 * File:	Mwesm_Armo.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordArmo class.
 *
 *=========================================================================*/
#ifndef __MWESM_ARMO_H
#define __MWESM_ARMO_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/


/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

  typedef enum {
	MWESM_ARMORTYPE_BOOTS	= 5,
	MWESM_ARMORTYPE_SHIELD	= 8
   } mwesmarmortype_t;

	/* Armor data structure */
  typedef struct {
    long  Type;
    float Weight;
    long  Value;
    long  Health;
    long  EnchantPts;
    long  Armour;
   } mwesmarmodata_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordArmo Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = Item ID, required
	MODL = Model Filename, required
	FNAM = Item Name, required
	AODT = Armour Data, required (24 bytes)
		
	ITEX = Icon Filename, required
	INDX = Body Part Index (1 byte)
	BNAM = Part Name
	CNAM = Female Body Part Name (0 occurences)
		INDX and BNAM/CNAM are grouped together.  INDX first followed
		by an optional BNAM (no BNAM indicates a NULL field for that index).
		Up to 7 pairs allowed.
	SCRI = Script Name
	ENAM = Enchantment Name
 *
 *=========================================================================*/
class CMwEsmRecordArmo : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString		m_ID;
  CMwEsmString		m_Model;
  CMwEsmString		m_Name;
  CMwEsmString		m_Icon;
  CMwEsmString		m_Enchantment;
  CMwEsmString		m_Script;

  mwesmarmodata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordArmo();
  virtual ~CMwEsmRecordArmo() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */
  const char*  GetTypeString    (void) const { return ("Armor"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordArmo Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

