/*===========================================================================
 *
 * File:	Mwesm_Clot.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordClot class.
 *
 *=========================================================================*/
#ifndef __MWESM_CLOT_H
#define __MWESM_CLOT_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/


/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

  typedef enum {
	MWESM_CLOTHTYPE_PANTS	= 0,
   } mwesmaclothtype_t;

	/* Armor data structure */
  typedef struct {
    long  Type;
    float Weight;
    short Value;
    short EnchantPts;		
   } mwesmclotdata_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordClot Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = Item ID, required
	MODL = Model Name, required
	FNAM = Item Name, required
	CTDT = Clothing Data (12 bytes), required
	ITEX = Inventory Icon
	INDX = Body Part Index (1 byte)
	BNAM = Male Body Part Name
	CNAM = Female Body Part Name
		INDX and BNAM/CNAM are grouped together.  INDX first followed
		by an optional BNAM (no BNAM indicates a NULL field for that index).
		Up to 7 pairs allowed.
	ENAM = Enchantment Name
	SCRI = Script Name
 *
 *=========================================================================*/
class CMwEsmRecordClot : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString		m_ID;
  CMwEsmString		m_Model;
  CMwEsmString		m_Name;
  CMwEsmString		m_Icon;
  CMwEsmString		m_Enchantment;
  CMwEsmString		m_Script;

  mwesmclotdata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordClot();
  virtual ~CMwEsmRecordClot() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */
  const char*  GetTypeString    (void) const { return ("Clothing"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordClot Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

