/*===========================================================================
 *
 * File:	Mwesm_Misc.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordMisc class.
 *
 *=========================================================================*/
#ifndef __MWESM_MISC_H
#define __MWESM_MISC_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/


/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)


	/* Repair data structure */
  typedef struct {
    float Weight;
    long  Value;
    long  Unknown;
   } mwesmmiscdata_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordMisc Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = item ID, required
	MODL = model filename, required
	FNAM = item name
	MCDT = Weapon Data, 12 bytes binary, required
	ITEX = Iventory icon filename
	ENAM = Enchantment ID string
	SCRI = script ID string
 *
 *=========================================================================*/
class CMwEsmRecordMisc : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString		m_ID;
  CMwEsmString		m_Model;
  CMwEsmString		m_Name;
  CMwEsmString		m_Icon;
  CMwEsmString		m_Enchantment;
  CMwEsmString		m_Script;

  mwesmmiscdata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordMisc();
  virtual ~CMwEsmRecordMisc() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */

  const char*  GetTypeString    (void) const { return ("Misc"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordMisc Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

