/*===========================================================================
 *
 * File:	Mwesm_Prob.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordProb class.
 *
 *=========================================================================*/
#ifndef __MWESM_PROB_H
#define __MWESM_PROB_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/


/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)


  typedef struct {
    float	Weight;
    long	Value;
    float	Quality;
    long 	Uses;
   } mwesmprobdata_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordProb Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = Item ID, required
	MODL = Model Name, required
	FNAM = Item Name, required
	PBDT = Probe Data (16 bytes), required
	ITEX = Inventory Icon
	SCRI = Script Name
 *
 *=========================================================================*/
class CMwEsmRecordProb : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString		m_ID;
  CMwEsmString		m_Model;
  CMwEsmString		m_Name;
  CMwEsmString		m_Icon;
  CMwEsmString		m_Enchantment;
  CMwEsmString		m_Script;

  mwesmprobdata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordProb();
  virtual ~CMwEsmRecordProb() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */
  const char*  GetTypeString    (void) const { return ("Probe"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordProb Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

