/*===========================================================================
 *
 * File:	Mwesm_Weap.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordWeap class.
 *
 *=========================================================================*/
#ifndef __MWESM_WEAP_H
#define __MWESM_WEAP_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Weapon flags (bit fields) */
  #define MWESM_WEAP_FLAG_NONE			0
  #define MWESM_WEAP_FLAG_IGNOREWEAPONRESIST	1

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

	/* Types of weapons, values should match with array in CPP file */
  typedef enum {
  	MWESM_WEAPTYPE_SHORT		= 0,	
	MWESM_WEAPTYPE_LONGONEHAND	= 1,
	MWESM_WEAPTYPE_LONGTWOHAND	= 2,
	MWESM_WEAPTYPE_BLUNTONEHAND	= 3,
	MWESM_WEAPTYPE_BLUNTTWOCLOSE	= 4,
	MWESM_WEAPTYPE_BLUNTTWOWIDE	= 5,
	MWESM_WEAPTYPE_SPEAR		= 6,
	MWESM_WEAPTYPE_AXEONEHAND	= 7,
	MWESM_WEAPTYPE_AXETWOHAND	= 8,
	MWESM_WEAPTYPE_BOW		= 9,
	MWESM_WEAPTYPE_CROSSBOW		= 10,
	MWESM_WEAPTYPE_THROWN		= 11,
	MWESM_WEAPTYPE_ARROW		= 12,
	MWESM_WEAPTYPE_BOLT		= 13 
    } mwesmweaptype_t;

	/* Weapon data structure */
  typedef struct {
    float	Weight;
    long	Value;
    short	Type;
    short	Health;
    float	Speed;
    float	Reach;
    short	Enchantment;
    byte	ChopMin;
    byte	ChopMax;
    byte	SlashMin;
    byte	SlashMax;
    byte	ThrustMin;
    byte	ThrustMax;
    long	Flags;
   } mwesmweapdata_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordWeap Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = item ID, required
	MODL = model filename, required
	FNAM = item name
	WPDT = Weapon Data, 0x20 bytes binary, required
		float Weight
		long  Value
		short Type? (0 to 13)
		short Health
		float Speed
		float Reach
		short Enchantment
		byte  ChopMin
		byte  ChopMax
		byte  SlashMin
		byte  SlashMax
		byte  ChopMin
		byte  ChopMax
		long  Flags (0 to 1)
	ITEX = Iventory icon filename
	ENAM = Enchantment ID string
	SCRI = script ID string
 *
 *=========================================================================*/
class CMwEsmRecordWeap : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString	m_ID;
  CMwEsmString	m_Model;
  CMwEsmString	m_Name;
  CMwEsmString	m_Icon;
  CMwEsmString	m_Enchantment;
  CMwEsmString	m_Script;

  mwesmweapdata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordWeap();
  virtual ~CMwEsmRecordWeap() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */
  const char*  GetTypeString    (void) const { return ("Weapon"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordWeap Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

