/*===========================================================================
 *
 * File:	MorrowMapLocTypes.JAVA
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Monday, August 27, 2001
 *
 * Defines the class for holding various location type properties such
 * as colors, zoom levels, etc...
 *
 *=========================================================================*/

	/* Import Packages */
import java.awt.*;
import java.io.*;


/*===========================================================================
 *
 * Begin Class MorrowMapLocTypes
 *
 * Holds arrays of information related to the location types.
 *
 *=========================================================================*/
public class MorrowMapLocTypes {

  /*---------- Class Constants -----------------------------------------*/

	/* Defines the maximum size of the arrays */
  static final int MAX_LOC_TYPES = 33;

	/* Default color indices for objects */
  static final Color TAMMAP_COLORS[] = {	
	new Color(0,0,0),		// 0  - Black 
	new Color(68,91,85),		// 1  - Water
	new Color(192,192,192),		// 2  - Province 1
	new Color(255,241,172),		// 3  - Province 2
	new Color(255,241,172),		// 4  - Province 3
	new Color(255,241,172),		// 5  - Province 4
	new Color(255,241,172),		// 6  - Province 5
	new Color(255,241,172),		// 7  - Province 6
	new Color(255,241,172),		// 8  - Province 7
	new Color(255,241,172),		// 9  - Province 8
	new Color(255,241,172),		// 10 - Province 9
	new Color(255,241,172),		// 11 - Province 10
	new Color(255,241,172),		// 12 - Border 1
	new Color(255,241,172),		// 13 - Border 2
	new Color(255,241,172),		// 14 - Border 3
	new Color(255,241,172),		// 15 - River
	new Color(255,241,172),		// 16 - Road 1
	new Color(255,241,172),		// 17 - Road 2
	new Color(255,241,172),		// 18 - Road 3
	new Color(255,241,172),		// 19 - City 1
	new Color(255,241,172),		// 20 - City 2
	new Color(255,241,172),		// 21 - City 3
	new Color(255,241,172),		// 22 - Ruin 1
	new Color(255,241,172),		// 23 - Ruin 2
	new Color(255,241,172),		// 24 - Ruin 3
	new Color(255,241,172),		// 25 - Misc 1
	new Color(255,241,172),		// 26 - Misc 2
	new Color(255,241,172),		// 27 - Misc 3
	new Color(255,241,172),		// 28 - Trade 1
	new Color(255,241,172),		// 29 - Trade 2
	new Color(30,30,230),		// 30 - Boat Travel
	new Color(230,30,30),		// 31 - Mages Guild Travel
	new Color(238,230,30),		// 32 - Silt Strider Travel
	new Color(255,255,255)		// 33 - White
    };

	/* Default color used in various places */
  static final Color DEFAULT_COLOR = new Color(255,255,255);
					
  /*---------- End of Class Constants --------------------------------*/


/*===========================================================================
 *
 * Method - Color GetColor (LocType);
 *
 * Returns the color associated with the given location type.  Returns a
 * default color if the input type is not valid.
 *
 *=========================================================================*/
static public Color GetColor (int LocType) {
	
	/* Ensure a valid location type */
  if (LocType < 0 || LocType >= MAX_LOC_TYPES) return (DEFAULT_COLOR);

  return (TAMMAP_COLORS[LocType]);
 }
/*===========================================================================
 *		End of Method GetColor()
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Default Color Methods
 *
 *=========================================================================*/
static public Color GetWaterColor   () { return (TAMMAP_COLORS[1]); }
static public Color GetDefaultColor () { return (DEFAULT_COLOR); }
/*===========================================================================
 *		End of Default Color Methods
 *=========================================================================*/




 }
/*===========================================================================
 *		End of Class MorrowMapLocTypes
 *=========================================================================*/


/*===========================================================================
 *		End of File MorrowMapLocTypes.JAVA
 *=========================================================================*/
