/*===========================================================================
 *
 * File:	Gengraphbase.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __GENGRAPHBASE_H
#define __GENGRAPHBASE_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "WinUtil.h"
  #include "dl_err.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/

 	/* General types of graphs */
  #define GENGRAPH_TYPE_DEFAULT 0
  #define GENGRAPH_TYPE_SCAN    1

	/* Default colors */
  #define GENGRAPH_COLOR_GRAY  DL_RGB(212, 208, 200)
  #define GENGRAPH_COLOR_BLACK DL_RGB(0,0,0)
  #define GENGRAPH_COLOR_WHITE DL_RGB(255,255,255)
  #define GENGRAPH_COLOR_RED   DL_RGB(255,0,0)
  #define GENGRAPH_COLOR_GREEN DL_RGB(0,192,0)
  #define GENGRAPH_COLOR_BLUE  DL_RGB(0,0,255)

	/* Standard graph data type */
  typedef float graphdata_t; 

/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CGenGraphBase Definition
 *
 * Description
 *
 *=========================================================================*/
class CGenGraphBase {
  
  /*---------- Begin Protected Class Members ----------------------*/
protected:
  int		m_Left;		/* Logical object extents */
  int		m_Right;
  int		m_Top;
  int		m_Bottom;

  bool		m_Visible;	/* Display object? */
  

  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CGenGraphBase();
  virtual ~CGenGraphBase() { Destroy(); }
  virtual void Destroy (void);

	/* Get class members */
  int  GetLeft   (void) const { return (m_Left); }
  int  GetRight  (void) const { return (m_Right); }
  int  GetTop    (void) const { return (m_Top); }
  int  GetBottom (void) const { return (m_Bottom); }
  int  GetWidth  (void) const { return (m_Right  - m_Left + 1); }
  int  GetHeight (void) const { return (m_Bottom - m_Top  + 1); }
  bool IsVisible (void) const { return (m_Visible); }

	/* Events */
  virtual int OnPaint (CDC* pDC) { return (0); }

	/* Set class members */
  void SetLeft    (const int  Value) { m_Left   = Value; }
  void SetRight   (const int  Value) { m_Right  = Value; }
  void SetTop     (const int  Value) { m_Top    = Value; }
  void SetBottom  (const int  Value) { m_Bottom = Value; }
  void SetWidth   (const int  Value) { m_Right  = Value + m_Left - 1; }
  void SetHeight  (const int  Value) { m_Bottom = Value + m_Top  - 1; }
  void SetVisible (const bool Flag)  { m_Visible = Flag; }

  void SetPosition (const int Left,  const int Top)    { SetLeft(Left);   SetTop(Top); }
  void SetSize     (const int Width, const int Height) { SetWidth(Width); SetHeight(Height); }

 };
/*===========================================================================
 *		End of Class CGenGraphBase Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Gengraphbase.H
 *=========================================================================*/

