/*===========================================================================
 *
 * File:	Gengraphdata.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __GENGRAPHDATA_H
#define __GENGRAPHDATA_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "GenGBase.h"
  #include "GenGAxis.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/

	/* Holds information on a data series for graph output */
  typedef struct {
    float*	m_pData;
    int		m_NumPoints;
    bool	m_Referenced;
    int		m_SkipBytes;
   } dataseries_t;

	/* Graph data buffer array size */
  #define GENGDATA_BUFFER_SIZE 1000

/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CGenGraphData Definition
 *
 *=========================================================================*/
class CGenGraphData : public CGenGraphBase {

  /*---------- Begin Protected Class Members --------------------*/
protected:
  CGenGraphAxis* m_pXAxis;		/* Graph axes */
  CGenGraphAxis* m_pYAxis;

  dataseries_t	 m_XData;		/* The graph data */
  dataseries_t	 m_YData;
  
  CPen*		 m_pFramePen;		/* Output options */
  CBrush*	 m_pBGBrush;
  COLORREF	 m_DataColor;
  CPen*		 m_pDataPen;
  bool		 m_DrawBG;

  int		 m_Type;

  bool		 m_HasLastPoint;	/* Scan members */
  bool		 m_DisplayPoints;
  bool		 m_AutoScroll;
  bool		 m_DisplayScan;
  bool		 m_ForceCleanScan;
  bool		 m_ScanForwards;
  bool		 m_LastScanForwards;
  bool		 m_NormalizeXAxis;
  float		 m_LastX;
  float		 m_LastY;
  float		 m_ScanX;
  COLORREF	 m_ScanColor;
  CPen*		 m_pScanPen;

  float		 m_XDataBuffer[GENGDATA_BUFFER_SIZE];
  float		 m_YDataBuffer[GENGDATA_BUFFER_SIZE];
  int		 m_NumBuffers;
  int		 m_BufferStartIndex;
  

  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Scan helper methods */
  void AutoScrollXAxis  (const float X, CDC* pDC);
  void ClearScan	(const float X, CDC* pDC);
  void DrawPointPriv    (const float X, const float Y, CDC* pDC);
  void DrawScan         (const float X, CDC* pDC);
  int  GetScanLocation  (const float X, const bool ScanForwards);
	
	/* Helper output methods */
  int DrawBG     (CDC* pDC);
  int DrawData   (CDC* pDC);
  int DrawBuffer (CDC* pDC);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CGenGraphData();
  virtual ~CGenGraphData();
  virtual void Destroy     (void);
  virtual void DestroyData (void);

	/* Adds a point to the data buffer */
  void AddBufferPoint (const float X, const float Y);

	/* Output a single point to the graph */
  void DrawPoint (const float X, const float Y, CDC* pDC);

	/* Get class members */
  CGenGraphAxis* GetXAxis         (void) { return (m_pXAxis); }
  CGenGraphAxis* GetYAxis         (void) { return (m_pYAxis); }
  bool		 GetDisplayPoints (void) { return (m_DisplayPoints); }

	/* Get class members */
  bool HasValidAxes (void) const { return (m_pXAxis != NULL && m_pYAxis != NULL); }
  bool HasValidData (void) const { return (m_YData.m_pData != NULL); }

  	/* Events */
  virtual int OnPaint (CDC* pDC);

	/* Resets the scan parameters */
  void ResetScan (void);

	/* Set class members */
  void SetAxes		 (CGenGraphAxis* pXAxis, CGenGraphAxis* pYAxis);
  void SetFloatData	 (float* pXData, float* pYData, const int NumPoints);
  void SetFloatDataEx	 (float* pXData, const int XSkipBytes, float* pYData, const int YSkipBytes, const int NumPoints);
  void SetDrawBG	 (const bool Flag) { m_DrawBG = Flag; }
  void SetDisplayScan	 (const bool Flag) { m_DisplayScan = Flag; }
  void SetGraphType	 (const int  Type) { m_Type = Type; }
  void SetNormalizeXData (const bool Flag) { m_NormalizeXAxis = Flag; }
  void SetLineColor	 (const COLORREF Color);
  void SetDisplayPoints  (const bool Flag) { m_DisplayPoints = Flag; }

 };
/*===========================================================================
 *		End of Class CGenGraphData Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Gengraphdata.H
 *=========================================================================*/
