/*===========================================================================
 *
 * File:	Gengraph.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description

	// Register a window class, application initialization 
  WNDCLASSEX ClassInfo;
  ClassInfo.cbSize = sizeof(ClassInfo);
  ClassInfo.style = 0;
  ClassInfo.lpfnWndProc = DefWindowProc;
  ClassInfo.cbClsExtra = 0;
  ClassInfo.cbWndExtra = 0;
  ClassInfo.hInstance = m_hInstance;
  ClassInfo.hIcon = NULL;
  ClassInfo.hCursor = NULL;
  ClassInfo.hbrBackground = NULL;
  ClassInfo.hIconSm = NULL;
  ClassInfo.lpszMenuName = NULL;
  ClassInfo.lpszClassName = "GenGraphClass";
  RegisterClassEx(&ClassInfo);

	// Create custom control on form, using above class name

	// Add class object to view class definition

	// Subclass the control, before view initialization 
  CWnd* pWnd = GetDlgItem(IDC_DATA_GRAPH);
  if (pWnd != NULL) m_Graph.SubclassWindow(pWnd->m_hWnd);

 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "GenGraph.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("GenGraph.cpp");

	/* Debug definitions */
#ifdef _DEBUG
  #define new DEBUG_NEW
  #undef THIS_FILE
  static char THIS_FILE[] = __FILE__;
#endif
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CGenGraph Message Map
 *
 *=========================================================================*/
#if !defined(__BORLANDC__)
BEGIN_MESSAGE_MAP(CGenGraph, CWnd)
  //{{AFX_MSG_MAP(CGenGraph)
  ON_WM_PAINT()
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif
/*===========================================================================
 *		End of Class CGenGraph Message Map
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Constructor
 *
 *=========================================================================*/
CGenGraph::CGenGraph() {
  m_Type = GENGRAPH_TYPE_DEFAULT;

  m_LeftMargin   = GENGRAPH_LEFT_MARGIN;
  m_RightMargin  = GENGRAPH_RIGHT_MARGIN;
  m_TopMargin    = GENGRAPH_TOP_MARGIN;
  m_BottomMargin = GENGRAPH_BOTTOM_MARGIN;

  m_NumAxes = 0;
  m_NumData = 0;
  m_UpdateFlag = GENGRAPH_UPDATE_ALL;

  m_BGColor  = GENGRAPH_COLOR_GRAY;
  m_pBGBrush = NULL;
  CreateBGBrush();
 }
/*===========================================================================
 *		End of Class CGenGraph Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Destructor
 *
 *=========================================================================*/
CGenGraph::~CGenGraph() {
  Destroy();
  DestroyPointer(m_pBGBrush);
 }
/*===========================================================================
 *		End of Class CGenGraph Destructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - void Destroy (void);
 *
 * Description
 *
 *=========================================================================*/
void CGenGraph::Destroy (void) {
  DEFINE_FUNCTION("CGenGraph::Destroy()");
  int Index;

	/* Delete all data series */
  for (Index = 0; Index < m_NumData; Index++) {
    DestroyPointer(m_pData[Index]);
   }

	/* Delete all axes */
  for (Index = 0; Index < m_NumAxes; Index++) {
    DestroyPointer(m_pAxes[Index]);
   }

  m_NumData = 0;
  m_NumAxes = 0;
 }
/*===========================================================================
 *		End of Class Method CGenGraph::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - void AdjustAxes (void);
 *
 *=========================================================================*/
void CGenGraph::AdjustAxes (void) {
  int Index;

  for (Index = 0; Index < m_NumAxes; Index++) {

    if (m_pAxes[Index]->IsXAxis()) {
      m_pAxes[Index]->SetPosition(m_LeftMargin, GetWndHeight() - m_BottomMargin - 1);
      m_pAxes[Index]->SetSize(GetWndWidth() - m_LeftMargin - m_RightMargin, m_BottomMargin+1);
     }
    else if (m_pAxes[Index]->IsInverted()) {
      m_pAxes[Index]->SetPosition(GetWndWidth() - m_RightMargin, m_TopMargin);
      m_pAxes[Index]->SetSize(m_LeftMargin+1, GetWndHeight() - m_TopMargin - m_BottomMargin);
     }
    else {
      m_pAxes[Index]->SetPosition(0, m_TopMargin);
      m_pAxes[Index]->SetSize(m_LeftMargin+1, GetWndHeight() - m_TopMargin - m_BottomMargin);
     }

    m_pAxes[Index]->ComputeCoorTransform();
   }

 }
/*===========================================================================
 *		End of Class Method CGenGraph::AdjustAxes()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - int AddXAxis (void);
 *
 *=========================================================================*/
int CGenGraph::AddXAxis (void) {
  DEFINE_FUNCTION("CGenGraph::AddXAxis()");

	/* Ensure not too many axes */
  if (m_NumData >= GENGRAPH_NUM_AXES) {
    ErrorHandler.AddError (ERR_MAXINDEX, "Maximum of %d data axes defined!", GENGRAPH_NUM_AXES);
    return (-1);
   }

  	/* Create and initialize the new axis */
  CreatePointer(m_pAxes[m_NumAxes], CGenXAxis);

  m_pAxes[m_NumAxes]->SetPosition(GENGRAPH_LEFT_MARGIN, GetWndHeight() - GENGRAPH_BOTTOM_MARGIN - 1);
  m_pAxes[m_NumAxes]->SetSize(GetWndWidth() - GENGRAPH_LEFT_MARGIN - GENGRAPH_RIGHT_MARGIN, GENGRAPH_BOTTOM_MARGIN+1);
  
  m_NumAxes++;
  return (m_NumAxes - 1);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::AddXAxis()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - int AddYAxis (void);
 *
 *=========================================================================*/
int CGenGraph::AddYAxis (void) {
  DEFINE_FUNCTION("CGenGraph::AddYAxis()");

	/* Ensure not too many axes */
  if (m_NumData >= GENGRAPH_NUM_AXES) {
    ErrorHandler.AddError (ERR_MAXINDEX, "Maximum of %d data axes defined!", GENGRAPH_NUM_AXES);
    return (-1);
   }

  	/* Create and initialize the new axis */
  CreatePointer(m_pAxes[m_NumAxes], CGenYAxis);
  m_pAxes[m_NumAxes]->SetPosition(0, GENGRAPH_TOP_MARGIN);
  m_pAxes[m_NumAxes]->SetSize(GENGRAPH_LEFT_MARGIN+1, GetWndHeight() - GENGRAPH_TOP_MARGIN - GENGRAPH_BOTTOM_MARGIN);

  m_NumAxes++;
  return (m_NumAxes - 1);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::AddYAxis()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - int AddData (XAxisIndex, YAxisIndex);
 *
 *=========================================================================*/
int CGenGraph::AddData (const int XAxisIndex, const int YAxisIndex) {
  DEFINE_FUNCTION("CGenGraph::AddData()");

	/* Ensure valid input */
  if (!IsValidAxisIndex(XAxisIndex) || !IsValidAxisIndex(YAxisIndex)   ||
	XAxisIndex == YAxisIndex    || !m_pAxes[XAxisIndex]->IsXAxis() || 
	!m_pAxes[YAxisIndex]->IsYAxis()) {
    ErrorHandler.AddError(ERR_BADINPUT, "Invalid axis indices!");
    return (-1);
   }

	/* Ensure not too many data series */
  if (m_NumData >= GENGRAPH_NUM_DATA) {
    ErrorHandler.AddError (ERR_MAXINDEX, "Maximum of %d data series defined!", GENGRAPH_NUM_DATA);
    return (-1);
   }

	/* Create and initialize the data series */
  CreatePointer(m_pData[m_NumData], CGenGraphData);
  m_pData[m_NumData]->SetAxes(m_pAxes[XAxisIndex], m_pAxes[YAxisIndex]);
  m_pData[m_NumData]->SetGraphType(m_Type);

	/* Set whether the graph series should display the scan line or not */
  if (m_Type == GENGRAPH_TYPE_SCAN) {
    if (FindAxisData(XAxisIndex) == NULL)
      m_pData[m_NumData]->SetDisplayScan(true);
    else
      m_pData[m_NumData]->SetDisplayScan(false);
   }

  m_NumData++;
  return (m_NumData - 1);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::AddData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - void CreateBGBrush (void);
 *
 *=========================================================================*/
void CGenGraph::CreateBGBrush (void) {
  DEFINE_FUNCTION("CGenGraph::CreateBGBrush()");

  DestroyArrayPointer(m_pBGBrush);
  m_pBGBrush = new CBrush(m_BGColor);
  ValidateNewPointer(m_pBGBrush);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::CreateBGBrush()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - void DrawBG (pDC);
 *
 *=========================================================================*/
void CGenGraph::DrawBG (CDC* pDC) {
#if defined(__BORLANDC__)
  pDC->FillRect(0, 0, GetWndWidth(), GetWndHeight(), *m_pBGBrush);
#else
  pDC->FillSolidRect(0, 0, GetWndWidth(), GetWndHeight(), m_BGColor);
#endif
 }
/*===========================================================================
 *		End of Class Method CGenGraph::DrawBG()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - bool DrawPoint (X, Y, GraphIndex);
 *
 * Attempt to output the given point to the specified graph.
 *
 *=========================================================================*/
bool CGenGraph::DrawPoint (const float X, const float Y, const int GraphIndex) {


  if (!IsValidDataIndex(GraphIndex)) return (false);

#if defined(__BORLANDC__)
  CDC DC(::GetDC(GetHandle()));
  m_pData[GraphIndex]->DrawPoint(X, Y, &DC);
#else
  CDC* pDC = GetDC();

  if (pDC != NULL) {
    m_pData[GraphIndex]->DrawPoint(X, Y, pDC);
    ReleaseDC(pDC);
   }
#endif

  return (true);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::DrawPoint()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - CGenGraphData* FindAxisData (AxisIndex);
 *
 * Find the first graph data using the given axis, or NULL.
 *
 *=========================================================================*/
CGenGraphData* CGenGraph::FindAxisData (const int AxisIndex) {
  int Index;

  for (Index = 0; Index < m_NumData; Index++) {
    if (m_pData[Index]->GetXAxis() == m_pAxes[AxisIndex]) return (m_pData[Index]);
    if (m_pData[Index]->GetYAxis() == m_pAxes[AxisIndex]) return (m_pData[Index]);
   }

	/* Not found */
  return (NULL);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::FindAxisData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - CGenGraphAxis* GetAxis (AxisIndex);
 *
 * Returns the axis with the given index, or NULL on any error.
 *
 *=========================================================================*/
CGenGraphAxis* CGenGraph::GetAxis (const int AxisIndex) {
  if (!IsValidAxisIndex(AxisIndex)) return (NULL);
  return (m_pAxes[AxisIndex]);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::GetAxis()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - CGenGraphData* GetData (DataIndex);
 *
 * Returns the data series with the given index, or NULL on any error.
 *
 *=========================================================================*/
CGenGraphData* CGenGraph::GetData (const int DataIndex) {
  if (!IsValidDataIndex(DataIndex)) return (NULL);
  return (m_pData[DataIndex]);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::GetData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - int GetWndWidth (void) const;
 *
 *=========================================================================*/
int CGenGraph::GetWndWidth (void) const {
  RECT WindowRect;

#if defined(__BORLANDC__)
  GetWindowRect(WindowRect);
#else
  GetWindowRect(&WindowRect);
#endif
  
  return (WindowRect.right - WindowRect.left);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::GetWndWidth()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - int GetWndHeight (void) const;
 *
 *=========================================================================*/
int CGenGraph::GetWndHeight (void) const {
  RECT WindowRect;

#if defined(__BORLANDC__)
  GetWindowRect(WindowRect);
#else
  GetWindowRect(&WindowRect);
#endif

  return (WindowRect.bottom - WindowRect.top);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::GetWndHeight()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Event - void OnPaint ();
 *
 *=========================================================================*/
void CGenGraph::OnPaint() {
#if defined(__BORLANDC__)
  CPaintDC dc(GetHandle());
#else
  CPaintDC dc(this);
#endif
  int	   Index;

  if (m_UpdateFlag & GENGRAPH_UPDATE_BG) {
    DrawBG(&dc);
   }

	/* Output the axes first */
  if (m_UpdateFlag & GENGRAPH_UPDATE_AXES) {
    for (Index = 0; Index < m_NumAxes; Index++) {
      m_pAxes[Index]->OnPaint(&dc);
     }
   }      

   	/* Output the data series */
  if (m_UpdateFlag & GENGRAPH_UPDATE_GRAPHS) {
    for (Index = 0; Index < m_NumData; Index++) {
      m_pData[Index]->OnPaint(&dc);
     }
   }

	/* Reset the update flag */
  m_UpdateFlag = GENGRAPH_UPDATE_ALL;
 }
/*===========================================================================
 *		End of Class Event CGenGraph::OnPaint()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - void ResetScan (void);
 *
 * Resets all scan graphs.
 *
 *=========================================================================*/
void CGenGraph::ResetScan (void) {
  int Index;

  for (Index = 0; Index < m_NumData; Index++) {
    m_pData[Index]->ResetScan();
   }

 }
/*===========================================================================
 *		End of Class Method CGenGraph::ResetScan()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - bool SetAxisCoor (AxisIndex, Start, End);
 *
 *=========================================================================*/
bool CGenGraph::SetAxisCoor (const int AxisIndex, const graphdata_t Start, 
			     const graphdata_t End) {

  if (!IsValidAxisIndex(AxisIndex)) {
    ErrorHandler.AddError(ERR_BADARRAYINDEX, "Invalid axis index %d!", AxisIndex);
    return (false);
   }

  m_pAxes[AxisIndex]->SetCoordinates(Start, End);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::SetAxisCoor()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - bool SetAxisLabel (AxisIndex, pLabel);
 *
 *=========================================================================*/
bool CGenGraph::SetAxisLabel (const int AxisIndex, const TCHAR* pLabel) {
 
  if (!IsValidAxisIndex(AxisIndex)) {
    ErrorHandler.AddError(ERR_BADARRAYINDEX, "Invalid axis index %d!", AxisIndex);
    return (false);
   }

  m_pAxes[AxisIndex]->SetLabel(pLabel);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::SetAxisLabel()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - bool SetFloatData (DataIndex, pXData, pYData, NumPoints);
 *
 *=========================================================================*/
bool CGenGraph::SetFloatData (const int DataIndex, float* pXData, float* pYData, 
			      const int NumPoints) {

	/* Ensure valid input */
  if (!IsValidDataIndex(DataIndex)) {
    ErrorHandler.AddError(ERR_BADINPUT, "Invalid data series index %d!", DataIndex);
    return (false);
   }

  m_pData[DataIndex]->SetFloatData(pXData, pYData, NumPoints);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::SetFloatData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraph Method - bool SetGraphColor (DataIndex, Color);
 *
 *=========================================================================*/
bool CGenGraph::SetGraphColor (const int DataIndex, COLORREF Color) {

	/* Ensure valid input */
  if (!IsValidDataIndex(DataIndex)) {
    ErrorHandler.AddError(ERR_BADINPUT, "Invalid data series index %d!", DataIndex);
    return (false);
   }

  m_pData[DataIndex]->SetLineColor(Color);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CGenGraph::SetGraphColor()
 *=========================================================================*/


