/*===========================================================================
 *
 * File:	Gengraph.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __GENGRAPH_H
#define __GENGRAPH_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "GenGAxis.h"
  #include "GenGData.h"
  #include "GenXAxis.h"
  #include "GenYAxis.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/
	
	/* Default array size for graph sub-objects */
  #define GENGRAPH_NUM_AXES 64
  #define GENGRAPH_NUM_DATA 32

	/* Default size/positions for axes */
  #define GENGRAPH_LEFT_MARGIN   40
  #define GENGRAPH_RIGHT_MARGIN  40
  #define GENGRAPH_TOP_MARGIN    20
  #define GENGRAPH_BOTTOM_MARGIN 30

	/* Flags used for updating the graph */
  #define GENGRAPH_UPDATE_NONE	 0x0000
  #define GENGRAPH_UPDATE_BG	 0x0001
  #define GENGRAPH_UPDATE_AXES	 0x0002
  #define GENGRAPH_UPDATE_GRAPHS 0x0004
  #define GENGRAPH_UPDATE_ALL    0xFFFF

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CGenGraph Definition
 *
 *=========================================================================*/
class CGenGraph : public CWnd {

  /*---------- Begin Protected Class Members -------------------*/
protected:
  CGenGraphData*	m_pData[GENGRAPH_NUM_DATA];
  CGenGraphAxis*	m_pAxes[GENGRAPH_NUM_AXES];
  int			m_NumData;
  int			m_NumAxes;

  int			m_LeftMargin;	
  int			m_RightMargin;
  int			m_TopMargin;
  int			m_BottomMargin;

  int			m_Type;		/* General graph type */
  long			m_UpdateFlag;	/* Type of update to perform */

  COLORREF		m_BGColor;
  CBrush*		m_pBGBrush;


  /*---------- Begin Protected Class Methods -------------------*/
protected:

	/* Creates the background brush using the current color */
  void CreateBGBrush (void);
  
	/* Helper drawing methods */
  void DrawBG (CDC* pDC);


  /*---------- Begin Public Class Methods ----------------------*/
public:

	/* Constructor/destructor */
  CGenGraph();
  virtual ~CGenGraph();
  virtual void Destroy (void);

	/* Set the size and position of axes to nicely fit window */
  void AdjustAxes (void);

	/* Add components */
  int AddXAxis (void);
  int AddYAxis (void);
  int AddData  (const int XAxisIndex, const int YAxisIndex);

	/* Output a point to a graph */
  bool DrawPoint (const float X, const float Y, const int GraphIndex);

	/* Find the given objects */
  CGenGraphData* FindAxisData (const int AxisIndex);
  CGenGraphData* GetData      (const int DataIndex);
  CGenGraphAxis* GetAxis      (const int AxisIndex);

	/* Get class members */
  int  GetWndWidth      (void)		  const;
  int  GetWndHeight     (void)		  const;
  bool IsValidAxisIndex (const int Index) const { return (Index >= 0 && Index < m_NumAxes); }
  bool IsValidDataIndex (const int Index) const { return (Index >= 0 && Index < m_NumData); }

	/* Used to reset a scan graph */
  void ResetScan (void);

	/* Set class members */
  void SetGraphType    (const int  Type)  { m_Type = Type; }
  void SetLeftMargin   (const int  Value) { m_LeftMargin   = Value; }
  void SetRightMargin  (const int  Value) { m_RightMargin  = Value; }
  void SetTopMargin    (const int  Value) { m_TopMargin    = Value; }
  void SetBottomMargin (const int  Value) { m_BottomMargin = Value; }
  void SetUpdateFlag   (const long Value) { m_UpdateFlag   = Value; }

	/* Set a data series source data */  
  bool SetFloatData   (const int DataIndex, float* pXData, float* pYData, const int NumPoints);
  bool SetGraphColor  (const int DataIndex, COLORREF Color);
  bool SetGraphDrawBG (const int DataIndex, const bool Flag) { if (!IsValidDataIndex(DataIndex)) return (false); m_pData[DataIndex]->SetDrawBG(Flag); return (true); }

	/* Set axis properties */
  bool SetAxisCoor  (const int AxisIndex, const graphdata_t Start, const graphdata_t End);
  bool SetAxisLabel (const int AxisIndex, const TCHAR* pLabel);


	/* ClassWizard generated virtual function overrides */
public:
  //{{AFX_VIRTUAL(CGenGraph)
  //}}AFX_VIRTUAL


	/* Generated message map functions */
protected:
  //{{AFX_MSG(CGenGraph)
  afx_msg void OnPaint();
  //}}AFX_MSG

  DECLARE_MESSAGE_MAP()

 };
/*===========================================================================
 *		End of Class CGenGraph Definition
 *=========================================================================*/


//{{AFX_INSERT_LOCATION}}
//}}AFX_INSERT_LOCATION

#endif
/*===========================================================================
 *		End of File Gengraph.H
 *=========================================================================*/
