/*===========================================================================
 *
 * File:	Genxaxis.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "genxaxis.h"
#include "dl_math.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("GenXAxis.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenXAxis Method - int DrawAxis (CDC* pDC);
 *
 *=========================================================================*/
int CGenXAxis::DrawAxis (CDC* pDC) {
  //DEFINE_FUNCTION("CGenXAxis::DrawAxis()");

  pDC->SelectObject(m_pAxisPen);

  pDC->MoveTo (GetLeft(), GetTop());
  pDC->LineTo (GetRight()+1, GetTop());

  return (0);
 }
/*===========================================================================
 *		End of Class Method CGenXAxis::DrawAxis()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenXAxis Method - int DrawLabel (pDC);
 *
 *=========================================================================*/
int CGenXAxis::DrawLabel (CDC* pDC) {
  CSize Extents;
  int   X;
  int   Y;

	/* Ignore if nothing to display */
  if (!m_DisplayLabel || m_pLabel == NULL) return (0);

  pDC->SelectObject(m_pTickFont);
  Extents = pDC->GetTextExtent(_T("0"), 1);
  Y = GetTop() + m_TickLength + Extents.cy;

  pDC->SelectObject(m_pLabelFont);
  Extents = pDC->GetTextExtent(m_pLabel);
  X = GetLeft() + ((GetWidth() - Extents.cx)/2);

  pDC->SetTextColor(m_LabelColor);
  pDC->TextOut(X + m_XLabelOffset, Y + m_YLabelOffset, m_pLabel);

  return (0);
 }
/*===========================================================================
 *		End of Class Method CGenXAxis::DrawLabel()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenXAxis Method - int DrawTicks (pDC);
 *
 *=========================================================================*/
int CGenXAxis::DrawTicks (CDC* pDC) {
  float NewAxisStart;
  float NewAxisEnd;
  float GraphX;
  CSize Extents;
  int   X;
  int   Y;
  TCHAR Buffer[64];

	/* Ignore if ticks are not to be displayed */
  if (!m_DisplayTicks || m_TickSpacing <= 0) return (0);

	/* Compute nice axis start/end */
  NewAxisStart = (float) floor(m_StartCoor/m_TickSpacing)*m_TickSpacing;
  NewAxisEnd   = (float) ceil(m_EndCoor/m_TickSpacing)*m_TickSpacing;
  pDC->SelectObject(m_pTickFont);
  pDC->SelectObject(m_pAxisPen);
  pDC->SelectObject(m_pBGBrush);
  GraphX = NewAxisStart;
  Y      = m_Top;

	/* Output each tick mark */
  while (GraphX <= NewAxisEnd + 0.5*m_TickSpacing) {

		/* Ensure that the first and last ticks are output */
    if (GraphX < m_StartCoor) {
      X = FastCoorToLogical(m_StartCoor);
      sprintf (Buffer, m_NumFormat, m_StartCoor);
     }
    else if (GraphX > m_EndCoor) {
      X = FastCoorToLogical(m_EndCoor);
      sprintf (Buffer, m_NumFormat, m_EndCoor);
     }
    else {
      X = FastCoorToLogical(GraphX);
      sprintf (Buffer, m_NumFormat, GraphX);
     }

		/* Output the tick mark */
    pDC->MoveTo(X, Y);
    pDC->LineTo(X, Y + m_TickLength);

		/* Output the tick label */
    Extents = pDC->GetTextExtent(Buffer);
    pDC->TextOut(X - (Extents.cx/2), Y + m_TickLength, Buffer);
    GraphX = GraphX + m_TickSpacing;
   } 

  return (0);
 }
/*===========================================================================
 *		End of Class Method CGenXAxis::DrawTicks()
 *=========================================================================*/
